# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'legendary.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QFormLayout, QGroupBox,
    QHBoxLayout, QLabel, QLineEdit, QPushButton,
    QSizePolicy, QSpacerItem, QSpinBox, QVBoxLayout,
    QWidget)

class Ui_LegendarySettings(object):
    def setupUi(self, LegendarySettings):
        if not LegendarySettings.objectName():
            LegendarySettings.setObjectName(u"LegendarySettings")
        LegendarySettings.resize(608, 420)
        LegendarySettings.setWindowTitle(u"LegendarySettings")
        self.legendary_layout = QHBoxLayout(LegendarySettings)
        self.legendary_layout.setObjectName(u"legendary_layout")
        self.left_layout = QVBoxLayout()
        self.left_layout.setObjectName(u"left_layout")
        self.install_dir_group = QGroupBox(LegendarySettings)
        self.install_dir_group.setObjectName(u"install_dir_group")
        self.install_dir_group.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
        self.install_dir_layout = QVBoxLayout(self.install_dir_group)
        self.install_dir_layout.setObjectName(u"install_dir_layout")

        self.left_layout.addWidget(self.install_dir_group)

        self.download_group = QGroupBox(LegendarySettings)
        self.download_group.setObjectName(u"download_group")
        self.download_group.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
        self.download_layout = QFormLayout(self.download_group)
        self.download_layout.setObjectName(u"download_layout")
        self.download_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.max_workers_label = QLabel(self.download_group)
        self.max_workers_label.setObjectName(u"max_workers_label")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_workers_label.sizePolicy().hasHeightForWidth())
        self.max_workers_label.setSizePolicy(sizePolicy)

        self.download_layout.setWidget(0, QFormLayout.LabelRole, self.max_workers_label)

        self.max_workers_layout = QHBoxLayout()
        self.max_workers_layout.setObjectName(u"max_workers_layout")
        self.max_worker_spin = QSpinBox(self.download_group)
        self.max_worker_spin.setObjectName(u"max_worker_spin")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.max_worker_spin.sizePolicy().hasHeightForWidth())
        self.max_worker_spin.setSizePolicy(sizePolicy1)
        self.max_worker_spin.setMinimum(0)
        self.max_worker_spin.setMaximum(16)
        self.max_worker_spin.setValue(0)

        self.max_workers_layout.addWidget(self.max_worker_spin)

        self.max_workers_info_label = QLabel(self.download_group)
        self.max_workers_info_label.setObjectName(u"max_workers_info_label")
        font = QFont()
        font.setItalic(True)
        self.max_workers_info_label.setFont(font)

        self.max_workers_layout.addWidget(self.max_workers_info_label)


        self.download_layout.setLayout(0, QFormLayout.ItemRole.FieldRole, self.max_workers_layout)

        self.max_memory_label = QLabel(self.download_group)
        self.max_memory_label.setObjectName(u"max_memory_label")

        self.download_layout.setWidget(1, QFormLayout.LabelRole, self.max_memory_label)

        self.max_memory_layout = QHBoxLayout()
        self.max_memory_layout.setObjectName(u"max_memory_layout")
        self.max_memory_spin = QSpinBox(self.download_group)
        self.max_memory_spin.setObjectName(u"max_memory_spin")
        sizePolicy1.setHeightForWidth(self.max_memory_spin.sizePolicy().hasHeightForWidth())
        self.max_memory_spin.setSizePolicy(sizePolicy1)
        self.max_memory_spin.setMinimum(0)
        self.max_memory_spin.setMaximum(10240)
        self.max_memory_spin.setSingleStep(128)
        self.max_memory_spin.setValue(1024)

        self.max_memory_layout.addWidget(self.max_memory_spin)

        self.max_memory_info_label = QLabel(self.download_group)
        self.max_memory_info_label.setObjectName(u"max_memory_info_label")
        self.max_memory_info_label.setFont(font)

        self.max_memory_layout.addWidget(self.max_memory_info_label)


        self.download_layout.setLayout(1, QFormLayout.ItemRole.FieldRole, self.max_memory_layout)

        self.preferred_cdn_label = QLabel(self.download_group)
        self.preferred_cdn_label.setObjectName(u"preferred_cdn_label")

        self.download_layout.setWidget(2, QFormLayout.LabelRole, self.preferred_cdn_label)

        self.preferred_cdn_line = QLineEdit(self.download_group)
        self.preferred_cdn_line.setObjectName(u"preferred_cdn_line")

        self.download_layout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.preferred_cdn_line)

        self.disable_https_label = QLabel(self.download_group)
        self.disable_https_label.setObjectName(u"disable_https_label")

        self.download_layout.setWidget(3, QFormLayout.LabelRole, self.disable_https_label)

        self.disable_https_check = QCheckBox(self.download_group)
        self.disable_https_check.setObjectName(u"disable_https_check")

        self.download_layout.setWidget(3, QFormLayout.ItemRole.FieldRole, self.disable_https_check)


        self.left_layout.addWidget(self.download_group)

        self.left_vspacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.left_layout.addItem(self.left_vspacer)


        self.legendary_layout.addLayout(self.left_layout)

        self.right_layout = QVBoxLayout()
        self.right_layout.setObjectName(u"right_layout")
        self.locale_group = QGroupBox(LegendarySettings)
        self.locale_group.setObjectName(u"locale_group")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.locale_group.sizePolicy().hasHeightForWidth())
        self.locale_group.setSizePolicy(sizePolicy2)
        self.locale_group.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
        self.locale_layout = QVBoxLayout(self.locale_group)
        self.locale_layout.setObjectName(u"locale_layout")

        self.right_layout.addWidget(self.locale_group)

        self.cleanup_group = QGroupBox(LegendarySettings)
        self.cleanup_group.setObjectName(u"cleanup_group")
        self.cleanup_group.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
        self.cleanup_layout = QVBoxLayout(self.cleanup_group)
        self.cleanup_layout.setObjectName(u"cleanup_layout")
        self.clean_keep_manifests_button = QPushButton(self.cleanup_group)
        self.clean_keep_manifests_button.setObjectName(u"clean_keep_manifests_button")

        self.cleanup_layout.addWidget(self.clean_keep_manifests_button)

        self.clean_button = QPushButton(self.cleanup_group)
        self.clean_button.setObjectName(u"clean_button")

        self.cleanup_layout.addWidget(self.clean_button)


        self.right_layout.addWidget(self.cleanup_group)

        self.metadata_group = QGroupBox(LegendarySettings)
        self.metadata_group.setObjectName(u"metadata_group")
        self.metadata_layout = QVBoxLayout(self.metadata_group)
        self.metadata_layout.setObjectName(u"metadata_layout")
        self.fetch_win32_check = QCheckBox(self.metadata_group)
        self.fetch_win32_check.setObjectName(u"fetch_win32_check")

        self.metadata_layout.addWidget(self.fetch_win32_check)

        self.fetch_macos_check = QCheckBox(self.metadata_group)
        self.fetch_macos_check.setObjectName(u"fetch_macos_check")

        self.metadata_layout.addWidget(self.fetch_macos_check)

        self.fetch_unreal_check = QCheckBox(self.metadata_group)
        self.fetch_unreal_check.setObjectName(u"fetch_unreal_check")

        self.metadata_layout.addWidget(self.fetch_unreal_check)

        self.exclude_non_asset_check = QCheckBox(self.metadata_group)
        self.exclude_non_asset_check.setObjectName(u"exclude_non_asset_check")

        self.metadata_layout.addWidget(self.exclude_non_asset_check)

        self.exclude_entitlements_check = QCheckBox(self.metadata_group)
        self.exclude_entitlements_check.setObjectName(u"exclude_entitlements_check")

        self.metadata_layout.addWidget(self.exclude_entitlements_check)

        self.metadata_info = QLabel(self.metadata_group)
        self.metadata_info.setObjectName(u"metadata_info")
        self.metadata_info.setFont(font)

        self.metadata_layout.addWidget(self.metadata_info)

        self.refresh_metadata_button = QPushButton(self.metadata_group)
        self.refresh_metadata_button.setObjectName(u"refresh_metadata_button")

        self.metadata_layout.addWidget(self.refresh_metadata_button)


        self.right_layout.addWidget(self.metadata_group)

        self.right_vspacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.right_layout.addItem(self.right_vspacer)


        self.legendary_layout.addLayout(self.right_layout)


        self.retranslateUi(LegendarySettings)
    # setupUi

    def retranslateUi(self, LegendarySettings):
        self.install_dir_group.setTitle(QCoreApplication.translate("LegendarySettings", u"Default installation folder", None))
        self.download_group.setTitle(QCoreApplication.translate("LegendarySettings", u"Download settings", None))
        self.max_workers_label.setText(QCoreApplication.translate("LegendarySettings", u"Max workers", None))
        self.max_workers_info_label.setText(QCoreApplication.translate("LegendarySettings", u"Less is slower (0: Default)", None))
        self.max_memory_label.setText(QCoreApplication.translate("LegendarySettings", u"Max shared memory", None))
        self.max_memory_spin.setSuffix(QCoreApplication.translate("LegendarySettings", u"MiB", None))
        self.max_memory_info_label.setText(QCoreApplication.translate("LegendarySettings", u"Less is slower (0: Default)", None))
        self.preferred_cdn_label.setText(QCoreApplication.translate("LegendarySettings", u"Preferred CDN", None))
        self.preferred_cdn_line.setPlaceholderText(QCoreApplication.translate("LegendarySettings", u"Default", None))
        self.disable_https_label.setText(QCoreApplication.translate("LegendarySettings", u"Disable HTTPS", None))
        self.disable_https_check.setText("")
        self.locale_group.setTitle(QCoreApplication.translate("LegendarySettings", u"Locale", None))
        self.cleanup_group.setTitle(QCoreApplication.translate("LegendarySettings", u"Cleanup", None))
        self.clean_keep_manifests_button.setText(QCoreApplication.translate("LegendarySettings", u"Clean, but keep manifests", None))
        self.clean_button.setText(QCoreApplication.translate("LegendarySettings", u"Remove everything", None))
        self.metadata_group.setTitle(QCoreApplication.translate("LegendarySettings", u"Platforms", None))
        self.fetch_win32_check.setText(QCoreApplication.translate("LegendarySettings", u"Include Win32 games", None))
        self.fetch_macos_check.setText(QCoreApplication.translate("LegendarySettings", u"Include macOS games", None))
        self.fetch_unreal_check.setText(QCoreApplication.translate("LegendarySettings", u"Include Unreal engine", None))
#if QT_CONFIG(tooltip)
        self.exclude_non_asset_check.setToolTip(QCoreApplication.translate("LegendarySettings", u"Do not load metadata for non-asset games (i.e. Origin games) on start-up.\n"
"\n"
"Disabling this greatly improves start-up time, but some games might not be visible in your library.", None))
#endif // QT_CONFIG(tooltip)
        self.exclude_non_asset_check.setText(QCoreApplication.translate("LegendarySettings", u"Exclude non-asset games", None))
#if QT_CONFIG(tooltip)
        self.exclude_entitlements_check.setToolTip(QCoreApplication.translate("LegendarySettings", u"Do not load entitlement data (i.e game's date of purchase) on start-up.\n"
"\n"
"Disabling this greatly improves start-up time, but some library filters may not work.", None))
#endif // QT_CONFIG(tooltip)
        self.exclude_entitlements_check.setText(QCoreApplication.translate("LegendarySettings", u"Exclude entitlements", None))
        self.metadata_info.setText(QCoreApplication.translate("LegendarySettings", u"Restart Rare to apply", None))
        self.refresh_metadata_button.setText(QCoreApplication.translate("LegendarySettings", u"Refresh metadata", None))
        pass
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'wishlist.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractScrollArea, QApplication, QCheckBox, QComboBox,
    QHBoxLayout, QLabel, QPushButton, QScrollArea,
    QSizePolicy, QSpacerItem, QVBoxLayout, QWidget)

class Ui_Wishlist(object):
    def setupUi(self, Wishlist):
        if not Wishlist.objectName():
            Wishlist.setObjectName(u"Wishlist")
        Wishlist.resize(489, 165)
        Wishlist.setWindowTitle(u"Wishlist")
        self.main_layout = QVBoxLayout(Wishlist)
        self.main_layout.setObjectName(u"main_layout")
        self.tool_layout = QHBoxLayout()
        self.tool_layout.setObjectName(u"tool_layout")
        self.filter_combo = QComboBox(Wishlist)
        self.filter_combo.setObjectName(u"filter_combo")

        self.tool_layout.addWidget(self.filter_combo)

        self.order_combo = QComboBox(Wishlist)
        self.order_combo.setObjectName(u"order_combo")

        self.tool_layout.addWidget(self.order_combo)

        self.reverse_check = QCheckBox(Wishlist)
        self.reverse_check.setObjectName(u"reverse_check")

        self.tool_layout.addWidget(self.reverse_check)

        self.hspacer_left = QSpacerItem(0, 0, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.tool_layout.addItem(self.hspacer_left)

        self.reload_button = QPushButton(Wishlist)
        self.reload_button.setObjectName(u"reload_button")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.reload_button.sizePolicy().hasHeightForWidth())
        self.reload_button.setSizePolicy(sizePolicy)

        self.tool_layout.addWidget(self.reload_button)


        self.main_layout.addLayout(self.tool_layout)

        self.scrollarea = QScrollArea(Wishlist)
        self.scrollarea.setObjectName(u"scrollarea")
        self.scrollarea.setSizeAdjustPolicy(QScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.scrollarea.setWidgetResizable(True)
        self.container = QWidget()
        self.container.setObjectName(u"container")
        self.container.setGeometry(QRect(0, 0, 473, 115))
        self.container_layout = QVBoxLayout(self.container)
        self.container_layout.setObjectName(u"container_layout")
        self.no_games_label = QLabel(self.container)
        self.no_games_label.setObjectName(u"no_games_label")

        self.container_layout.addWidget(self.no_games_label, 0, Qt.AlignmentFlag.AlignTop)

        self.scrollarea.setWidget(self.container)

        self.main_layout.addWidget(self.scrollarea)


        self.retranslateUi(Wishlist)
    # setupUi

    def retranslateUi(self, Wishlist):
        self.reverse_check.setText(QCoreApplication.translate("Wishlist", u"Reverse", None))
        self.reload_button.setText("")
        self.no_games_label.setText(QCoreApplication.translate("Wishlist", u"No games matching your filter", None))
        pass
    # retranslateUi




import difflib
import os
from datetime import datetime
from enum import Enum
from logging import getLogger
from typing import Tuple

import orjson
import requests

from rare.lgndr.core import LegendaryCore
from rare.utils.paths import cache_dir

logger = getLogger("SteamGrades")

replace_chars = ",;.:-_ "
steamapi_url = "https://api.steampowered.com/ISteamApps/GetAppList/v2/"
protondb_url = "https://www.protondb.com/api/v1/reports/summaries/"

__steam_ids_json = None
__grades_json = None
__active_download = False


class ProtondbRatings(int, Enum):
    # internal
    PENDING = ("pending", -2)
    FAIL = ("fail", -1)
    # protondb
    NA = ("na", 0)
    BORKED = ("borked", 1)
    BRONZE = ("bronze", 2)
    SILVER = ("silver", 3)
    GOLD = ("gold", 4)
    PLATINUM = ("platinum", 5)

    def __new__(cls, name: str, value: int):
        obj = int.__new__(cls, value)
        obj._value_ = value
        obj._name_ = name
        return obj

    def __str__(self):
        return self._name_

    def __int__(self):
        return self._value_


def get_rating(core: LegendaryCore, app_name: str) -> Tuple[int, str]:
    game = core.get_game(app_name)
    try:
        steam_id = get_steam_id(game.app_title)
        if not steam_id:
            raise Exception
        grade = get_grade(steam_id)
    except Exception:
        logger.error("Failed to get ProtonDB rating for %s", game.app_title)
        return 0, "fail"
    else:
        return steam_id, grade


# you should initiate the module with the game's steam code
def get_grade(steam_code):
    if steam_code == 0:
        return "fail"
    steam_code = str(steam_code)
    res = requests.get(f"{protondb_url}{steam_code}.json")
    try:
        app = orjson.loads(res.text)
    except orjson.JSONDecodeError as e:
        logger.error("Failed to get ProtonDB response for %s", steam_code)
        return "fail"

    return app.get("tier", "fail")


def load_json() -> dict:
    global __active_download
    if __active_download:
        return {}
    file = os.path.join(cache_dir(), "steam_appids.json")
    elapsed_days = 0
    if os.path.exists(file):
        mod_time = datetime.fromtimestamp(os.path.getmtime(file))
        elapsed_days = abs(datetime.now() - mod_time).days
    if not os.path.exists(file) or elapsed_days > 7:
        __active_download = True
        response = requests.get(steamapi_url)
        __active_download = False
        apps = orjson.loads(response.text).get("applist", {}).get("apps", {})
        ids = {}
        for app in apps:
            ids[app["name"]] = app["appid"]
        with open(file, "w", encoding="utf-8") as f:
            f.write(orjson.dumps(ids).decode("utf-8"))
        return ids
    else:
        return orjson.loads(open(file, "r").read())


def get_steam_id(title: str) -> int:
    # workarounds for satisfactory
    # FIXME: This has to be made smarter.
    title = title.replace("Early Access", "").replace("Experimental", "").strip()
    # title = title.split(":")[0]
    # title = title.split("-")[0]
    global __steam_ids_json
    if __steam_ids_json is None:
        __steam_ids_json = load_json()
    ids = __steam_ids_json

    if title in ids.keys():
        steam_name = [title]
    else:
        steam_name = difflib.get_close_matches(title, ids.keys(), n=1, cutoff=0.5)
    if steam_name:
        return ids[steam_name[0]]
    else:
        return 0



from typing import List, Union

from PySide6.QtCore import Qt, Signal
from PySide6.QtWidgets import QCheckBox, QWidget, QVBoxLayout
from legendary.utils.selective_dl import get_sdl_appname

from rare.models.game import RareGame


class TagCheckBox(QCheckBox):
    def __init__(self, text, desc, tags: List[str], parent=None):
        super(TagCheckBox, self).__init__(parent)
        self.setText(text)
        self.setToolTip(desc)
        self.tags = tags

    def isChecked(self) -> Union[bool, List[str]]:
        return self.tags if super(TagCheckBox, self).isChecked() else False


class SelectiveWidget(QWidget):
    stateChanged: Signal = Signal()

    def __init__(self, rgame: RareGame, platform: str, parent=None):
        super().__init__(parent=parent)

        main_layout = QVBoxLayout(self)
        main_layout.setSpacing(0)

        core = rgame.core

        config_tags = core.lgd.config.get(rgame.app_name, "install_tags", fallback=None)
        config_disable_sdl = core.lgd.config.getboolean(rgame.app_name, "disable_sdl", fallback=False)
        sdl_name = get_sdl_appname(rgame.app_name)
        if not config_disable_sdl and sdl_name is not None:
            sdl_data = core.get_sdl_data(sdl_name, platform=platform)
            if sdl_data:
                for tag, info in sdl_data.items():
                    cb = TagCheckBox(info["name"].strip(), info["description"].strip(), info["tags"])
                    if tag == "__required":
                        cb.setChecked(True)
                        cb.setDisabled(True)
                    if config_tags is not None:
                        if all(elem in config_tags for elem in info["tags"]):
                            cb.setChecked(True)
                    cb.stateChanged.connect(self.stateChanged)
                    main_layout.addWidget(cb)
            self.parent().setDisabled(False)
        else:
            self.parent().setDisabled(True)

    def install_tags(self):
        install_tags = [""]
        for cb in self.findChildren(TagCheckBox, options=Qt.FindChildOption.FindDirectChildrenOnly):
            if data := cb.isChecked():
                # noinspection PyTypeChecker
                install_tags.extend(data)
        return install_tags



from PySide6.QtCore import QSize
from PySide6.QtGui import QPainter
from PySide6.QtWidgets import QProxyStyle, QTabBar, QStyle, QStyleOption, QWidget, QStyleOptionTab


# NOTE: possible styling conflicts
#
#       SideTabBar
#


class RareStyle(QProxyStyle):

    def sizeFromContents(self, types: QStyle.ContentsType, option: QStyleOption, size: QSize, widget: QWidget) -> QSize:
        size = super(RareStyle, self).sizeFromContents(types, option, size, widget)
        if types == QStyle.ContentsType.CT_TabBarTab:
            if option.shape in {QTabBar.Shape.RoundedEast, QTabBar.Shape.RoundedWest}:
                size.transpose()
        return size

    def drawControl(self, element: QStyle.ControlElement, option: QStyleOption, painter: QPainter, widget: QWidget) -> None:
        if element == QStyle.ControlElement.CE_TabBarTabLabel:
            if option.shape in {QTabBar.Shape.RoundedEast, QTabBar.Shape.RoundedWest}:
                opt = QStyleOptionTab(option)
                opt.shape = QTabBar.Shape.RoundedNorth
                super(RareStyle, self).drawControl(element, opt, painter, widget)
                return
        super(RareStyle, self).drawControl(element, option, painter, widget)



import os
import locale
from logging import getLogger

from PySide6.QtCore import QSettings, Qt, Slot, QUrl
from PySide6.QtGui import QDesktopServices
from PySide6.QtWidgets import QWidget, QMessageBox

from rare.components.tabs.settings.widgets.discord_rpc import DiscordRPCSettings
from rare.models.options import options, LibraryView
from rare.shared import LegendaryCoreSingleton
from rare.ui.components.tabs.settings.rare import Ui_RareSettings
from rare.utils.misc import (
    get_translations,
    get_color_schemes,
    set_color_pallete,
    get_style_sheets,
    set_style_sheet,
    format_size,
)
from rare.utils.paths import create_desktop_link, desktop_link_path, log_dir, desktop_links_supported

logger = getLogger("RareSettings")


class RareSettings(QWidget):
    def __init__(self, parent=None):
        super(RareSettings, self).__init__(parent=parent)
        self.ui = Ui_RareSettings()
        self.ui.setupUi(self)
        self.core = LegendaryCoreSingleton()
        self.settings = QSettings(self)

        # Select lang
        self.ui.lang_select.addItem(self.tr("System default"), options.language.default)
        for lang_code, title in get_translations():
            self.ui.lang_select.addItem(title, lang_code)
        language = self.settings.value(*options.language)
        if (index := self.ui.lang_select.findData(language, Qt.ItemDataRole.UserRole)) > 0:
            self.ui.lang_select.setCurrentIndex(index)
        else:
            self.ui.lang_select.setCurrentIndex(0)
        self.ui.lang_select.currentIndexChanged.connect(self.on_lang_changed)

        self.ui.color_select.addItem(self.tr("None"), "")
        for item in get_color_schemes():
            self.ui.color_select.addItem(item, item)
        color = self.settings.value(*options.color_scheme)
        if (index := self.ui.color_select.findData(color, Qt.ItemDataRole.UserRole)) > 0:
            self.ui.color_select.setCurrentIndex(index)
            self.ui.color_select.setDisabled(False)
            self.ui.style_select.setDisabled(True)
        else:
            self.ui.color_select.setCurrentIndex(0)
        self.ui.color_select.currentIndexChanged.connect(self.on_color_select_changed)

        self.ui.style_select.addItem(self.tr("None"), "")
        for item in get_style_sheets():
            self.ui.style_select.addItem(item, item)
        style = self.settings.value(*options.style_sheet)
        if (index := self.ui.style_select.findData(style, Qt.ItemDataRole.UserRole)) > 0:
            self.ui.style_select.setCurrentIndex(index)
            self.ui.style_select.setDisabled(False)
            self.ui.color_select.setDisabled(True)
        else:
            self.ui.style_select.setCurrentIndex(0)
        self.ui.style_select.currentIndexChanged.connect(self.on_style_select_changed)

        self.ui.view_combo.addItem(self.tr("Game covers"), LibraryView.COVER)
        self.ui.view_combo.addItem(self.tr("Vertical list"), LibraryView.VLIST)
        view = LibraryView(self.settings.value(*options.library_view))
        if (index := self.ui.view_combo.findData(view)) > -1:
            self.ui.view_combo.setCurrentIndex(index)
        else:
            self.ui.view_combo.setCurrentIndex(0)
        self.ui.view_combo.currentIndexChanged.connect(self.on_view_combo_changed)

        self.discord_rpc_settings = DiscordRPCSettings(self)
        self.ui.right_layout.insertWidget(1, self.discord_rpc_settings, alignment=Qt.AlignmentFlag.AlignTop)

        self.ui.sys_tray.setChecked(self.settings.value(*options.sys_tray))
        self.ui.sys_tray.stateChanged.connect(
            lambda: self.settings.setValue(options.sys_tray.key, self.ui.sys_tray.isChecked())
        )

        self.ui.auto_update.setChecked(self.settings.value(*options.auto_update))
        self.ui.auto_update.stateChanged.connect(
            lambda: self.settings.setValue(options.auto_update.key, self.ui.auto_update.isChecked())
        )

        self.ui.confirm_start.setChecked(self.settings.value(*options.confirm_start))
        self.ui.confirm_start.stateChanged.connect(
            lambda: self.settings.setValue(options.confirm_start.key, self.ui.confirm_start.isChecked())
        )
        # TODO: implement use when starting game, disable for now
        self.ui.confirm_start.setDisabled(True)

        self.ui.auto_sync_cloud.setChecked(self.settings.value(*options.auto_sync_cloud))
        self.ui.auto_sync_cloud.stateChanged.connect(
            lambda: self.settings.setValue(options.auto_sync_cloud.key, self.ui.auto_sync_cloud.isChecked())
        )

        self.ui.notification.setChecked(self.settings.value(*options.notification))
        self.ui.notification.stateChanged.connect(
            lambda: self.settings.setValue(options.notification.key, self.ui.notification.isChecked())
        )

        self.ui.save_size.setChecked(self.settings.value(*options.restore_window))
        self.ui.save_size.stateChanged.connect(self.save_window_size)

        self.ui.log_games.setChecked(self.settings.value(*options.log_games))
        self.ui.log_games.stateChanged.connect(
            lambda: self.settings.setValue(options.log_games.key, self.ui.log_games.isChecked())
        )

        if desktop_links_supported():
            self.desktop_link = desktop_link_path("Rare", "desktop")
            self.start_menu_link = desktop_link_path("Rare", "start_menu")
        else:
            self.ui.desktop_link_btn.setToolTip(self.tr("Not supported"))
            self.ui.desktop_link_btn.setDisabled(True)
            self.ui.startmenu_link_btn.setToolTip(self.tr("Not supported"))
            self.ui.startmenu_link_btn.setDisabled(True)
            self.desktop_link = ""
            self.start_menu_link = ""

        if self.desktop_link and self.desktop_link.exists():
            self.ui.desktop_link_btn.setText(self.tr("Remove desktop link"))

        if self.start_menu_link and self.start_menu_link.exists():
            self.ui.startmenu_link_btn.setText(self.tr("Remove start menu link"))

        self.ui.desktop_link_btn.clicked.connect(self.create_desktop_link)
        self.ui.startmenu_link_btn.clicked.connect(self.create_start_menu_link)

        self.ui.log_dir_open_button.clicked.connect(self.open_directory)
        self.ui.log_dir_clean_button.clicked.connect(self.clean_logdir)

        # get size of logdir
        size = sum(
            log_dir().joinpath(f).stat().st_size
            for f in log_dir().iterdir()
            if log_dir().joinpath(f).is_file()
        )
        self.ui.log_dir_size_label.setText(format_size(size))
        # self.log_dir_clean_button.setVisible(False)
        # self.log_dir_size_label.setVisible(False)

    @Slot()
    def clean_logdir(self):
        for f in log_dir().iterdir():
            try:
                if log_dir().joinpath(f).is_file():
                    log_dir().joinpath(f).unlink()
            except PermissionError as e:
                logger.error(e)
        size = sum(
            log_dir().joinpath(f).stat().st_size
            for f in log_dir().iterdir()
            if log_dir().joinpath(f).is_file()
        )
        self.ui.log_dir_size_label.setText(format_size(size))

    @Slot()
    def create_start_menu_link(self):
        try:
            if not os.path.exists(self.start_menu_link):
                if not create_desktop_link(app_name="rare_shortcut", link_type="start_menu"):
                    return
                self.ui.startmenu_link_btn.setText(self.tr("Remove start menu link"))
            else:
                os.remove(self.start_menu_link)
                self.ui.startmenu_link_btn.setText(self.tr("Create start menu link"))
        except PermissionError as e:
            logger.error(str(e))
            QMessageBox.warning(
                self,
                self.tr("Error"),
                self.tr("Permission error, cannot remove {}").format(self.start_menu_link),
            )

    @Slot()
    def create_desktop_link(self):
        try:
            if not os.path.exists(self.desktop_link):
                if not create_desktop_link(app_name="rare_shortcut", link_type="desktop"):
                    return
                self.ui.desktop_link_btn.setText(self.tr("Remove Desktop link"))
            else:
                os.remove(self.desktop_link)
                self.ui.desktop_link_btn.setText(self.tr("Create desktop link"))
        except PermissionError as e:
            logger.error(str(e))
            logger.warning(
                self,
                self.tr("Error"),
                self.tr("Permission error, cannot remove {}").format(self.start_menu_link),
            )

    @Slot(int)
    def on_color_select_changed(self, index: int):
        scheme = self.ui.color_select.itemData(index, Qt.ItemDataRole.UserRole)
        if scheme:
            self.ui.style_select.setCurrentIndex(0)
            self.ui.style_select.setDisabled(True)
        else:
            self.ui.style_select.setDisabled(False)
        self.settings.setValue("color_scheme", scheme)
        set_color_pallete(scheme)

    @Slot(int)
    def on_style_select_changed(self, index: int):
        style = self.ui.style_select.itemData(index, Qt.ItemDataRole.UserRole)
        if style:
            self.ui.color_select.setCurrentIndex(0)
            self.ui.color_select.setDisabled(True)
        else:
            self.ui.color_select.setDisabled(False)
        self.settings.setValue("style_sheet", style)
        set_style_sheet(style)

    @Slot(int)
    def on_view_combo_changed(self, index: int):
        view = LibraryView(self.ui.view_combo.itemData(index, Qt.ItemDataRole.UserRole))
        self.settings.setValue(options.library_view.key, int(view))

    @Slot()
    def open_directory(self):
        QDesktopServices.openUrl(QUrl(f"file://{log_dir()}"))

    @Slot()
    def save_window_size(self):
        self.settings.setValue(options.restore_window.key, self.ui.save_size.isChecked())
        self.settings.remove(options.window_width.key)
        self.settings.remove(options.window_height.key)

    @Slot(int)
    def on_lang_changed(self, index: int):
        lang_code = self.ui.lang_select.itemData(index, Qt.ItemDataRole.UserRole)
        if lang_code == locale.getlocale()[0]:
            self.settings.remove(options.language.key)
        else:
            self.settings.setValue(options.language.key, lang_code)



import os
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import List, Optional, Dict, Tuple

from legendary.models.downloading import AnalysisResult, ConditionCheckResult
from legendary.models.game import Game, InstalledGame

from rare.lgndr.downloader.mp.manager import DLManager


@dataclass
class InstallOptionsModel:
    app_name: str
    base_path: str = ""
    shared_memory: int = 1024
    max_workers: int = os.cpu_count() * 2
    force: bool = False
    platform: str = "Windows"
    install_tag: Optional[List[str]] = None
    order_opt: bool = False
    repair_mode: bool = False
    repair_and_update: bool = False
    no_install: bool = False
    ignore_space: bool = False
    reset_sdl: bool = False
    skip_dlcs: bool = False
    with_dlcs: bool = False
    # Rare's internal arguments
    # FIXME: Do we really need all of these?
    create_shortcut: bool = True
    overlay: bool = False
    update: bool = False
    silent: bool = False
    install_prereqs: bool = False

    def as_install_kwargs(self) -> Dict:
        return {
            k: getattr(self, k)
            for k in vars(self)
            if k not in ["update", "silent", "create_shortcut", "overlay", "install_prereqs"]
        }


@dataclass
class InstallDownloadModel:
    dlm: DLManager
    analysis: AnalysisResult
    igame: InstalledGame
    game: Game
    repair: bool
    repair_file: str
    res: ConditionCheckResult


class InstallQueueItemModel:
    def __init__(self, options: InstallOptionsModel,  download: InstallDownloadModel = None):
        self.options: Optional[InstallOptionsModel] = options
        # lk: internal attribute holders
        self.__download: Optional[InstallDownloadModel] = None
        self.__date: Optional[datetime] = None

        self.download = download

    @property
    def download(self) -> Optional[InstallDownloadModel]:
        return self.__download

    @download.setter
    def download(self, download: Optional[InstallDownloadModel]):
        self.__download = download
        if download is not None:
            self.__date = datetime.now()

    @property
    def expired(self) -> bool:
        return datetime.now() > (self.__date + timedelta(minutes=30))

    def __bool__(self):
        return (self.download is not None) and (self.options is not None) and (not self.expired)


@dataclass
class UninstallOptionsModel:
    app_name: str
    accepted: bool = None
    keep_files: bool = None
    keep_folder: bool = True
    keep_config: bool = None
    keep_overlay_keys: bool = None

    @property
    def __values(self) -> Tuple[bool, bool, bool, bool, bool]:
        return self.accepted, self.keep_files, self.keep_folder, self.keep_config, self.keep_overlay_keys

    @__values.setter
    def __values(self, values: Tuple[bool, bool, bool, bool, bool]):
        self.accepted, self.keep_files, self.keep_folder, self.keep_config, self.keep_overlay_keys = values

    def __bool__(self):
        return bool(self.app_name) and all(map(lambda x: x is not None, self.__values))

    def __iter__(self):
        return iter(self.__values)

    def set_accepted(self, keep_files, keep_folder, keep_config, keep_overlay_keys):
        self.__values = True, keep_files, keep_folder, keep_config, keep_overlay_keys

    def set_rejected(self):
        self.__values = False, None, None, None, None


@dataclass
class SelectiveDownloadsModel:
    app_name: str
    accepted: bool = None
    install_tag: Optional[List[str]] = None

    def __bool__(self):
        return (
            bool(self.app_name)
            and (self.accepted is not None)
            and (self.install_tag is not None)
        )


@dataclass
class MoveGameModel:
    app_name: str
    accepted: bool = None
    target_path: Optional[str] = None

    def __bool__(self):
        return (
            bool(self.app_name)
            and (self.accepted is not None)
            and (self.target_path is not None)
        )



import platform
import random
from abc import abstractmethod
from logging import getLogger

from PySide6.QtCore import Signal, Qt, Slot, QObject, QEvent
from PySide6.QtGui import QMouseEvent, QShowEvent, QPaintEvent, QAction
from PySide6.QtWidgets import QMessageBox

from rare.models.game import RareGame
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton, ArgumentsSingleton, ImageManagerSingleton
from rare.utils.paths import desktop_links_supported, desktop_link_path, create_desktop_link
from rare.utils.steam_shortcuts import (
    steam_shortcuts_supported,
    steam_shortcut_exists,
    remove_steam_shortcut,
    remove_steam_coverart,
    add_steam_shortcut,
    add_steam_coverart,
    save_steam_shortcuts,
)
from .library_widget import LibraryWidget

logger = getLogger("GameWidget")


class GameWidget(LibraryWidget):
    show_info = Signal(RareGame)

    def __init__(self, rgame: RareGame, parent=None):
        super(GameWidget, self).__init__(parent=parent)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()
        self.image_manager = ImageManagerSingleton()

        self.rgame: RareGame = rgame

        self.setContextMenuPolicy(Qt.ContextMenuPolicy.ActionsContextMenu)

        self.launch_action = QAction(self.tr("Launch"), self)
        self.launch_action.triggered.connect(self._launch)

        self.install_action = QAction(self.tr("Install"), self)
        self.install_action.triggered.connect(self._install)

        self.desktop_link_action = QAction(self)
        self.desktop_link_action.triggered.connect(lambda: self._create_link(self.rgame.folder_name, "desktop"))

        self.menu_link_action = QAction(self)
        self.menu_link_action.triggered.connect(lambda: self._create_link(self.rgame.folder_name, "start_menu"))

        self.steam_shortcut_action = QAction(self)
        self.steam_shortcut_action.triggered.connect(
            lambda: self._create_steam_shortcut(self.rgame.app_name, self.rgame.app_title)
        )

        self.reload_action = QAction(self.tr("Reload Image"), self)
        self.reload_action.triggered.connect(self._on_reload_image)

        self.uninstall_action = QAction(self.tr("Uninstall"), self)
        self.uninstall_action.triggered.connect(self._uninstall)

        self.update_actions()

        # signals
        self.rgame.signals.widget.update.connect(self.update_pixmap)
        self.rgame.signals.widget.update.connect(self.update_buttons)
        self.rgame.signals.widget.update.connect(self.update_state)
        self.rgame.signals.game.installed.connect(self.update_actions)
        self.rgame.signals.game.uninstalled.connect(self.update_actions)

        self.rgame.signals.progress.start.connect(self.start_progress)
        self.rgame.signals.progress.update.connect(lambda p: self.updateProgress(p))
        self.rgame.signals.progress.finish.connect(lambda e: self.hideProgress(e))

        self.state_strings = {
            RareGame.State.IDLE: "",
            RareGame.State.RUNNING: self.tr("Running..."),
            RareGame.State.DOWNLOADING: self.tr("Downloading..."),
            RareGame.State.VERIFYING: self.tr("Verifying..."),
            RareGame.State.MOVING: self.tr("Moving..."),
            RareGame.State.UNINSTALLING: self.tr("Uninstalling..."),
            RareGame.State.SYNCING: self.tr("Syncing saves..."),
            "has_update": self.tr("Update available"),
            "needs_verification": self.tr("Needs verification"),
            "not_can_launch": self.tr("Can't launch"),
            "save_not_up_to_date": self.tr("Save is not up-to-date"),
        }

        self.hover_strings = {
            "info": self.tr("Show information"),
            "install": self.tr("Install game"),
            "can_launch": self.tr("Launch game"),
            "is_foreign": self.tr("Launch offline"),
            "has_update": self.tr("Launch without version check"),
            "is_origin": self.tr("Launch/Link"),
            "not_can_launch": self.tr("Can't launch"),
        }

        self._ui = None

    # lk: abstract class for typing, the `self.ui` attribute should be used
    # lk: by the Ui class in the children. It must contain at least the same
    # lk: attributes as `GameWidgetUi` class

    @property
    def ui(self):
        return self._ui

    @ui.setter
    def ui(self, arg):
        self._ui = arg

    @abstractmethod
    def update_pixmap(self):
        pass

    @abstractmethod
    def start_progress(self):
        pass

    def paintEvent(self, a0: QPaintEvent) -> None:
        if not self.visibleRegion().isNull() and not self.rgame.has_pixmap:
            self.startTimer(random.randrange(42, 2361, 129), Qt.TimerType.CoarseTimer)
            # self.startTimer(random.randrange(42, 2361, 363), Qt.VeryCoarseTimer)
            # self.rgame.load_pixmap()
        super().paintEvent(a0)

    def timerEvent(self, a0):
        self.killTimer(a0.timerId())
        self.rgame.load_pixmaps()

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        super().showEvent(a0)

    @Slot()
    def update_state(self):
        if self.rgame.is_idle:
            if self.rgame.has_update:
                self.ui.status_label.setText(self.state_strings["has_update"])
            elif self.rgame.needs_verification:
                self.ui.status_label.setText(self.state_strings["needs_verification"])
            elif not self.rgame.can_launch and self.rgame.is_installed:
                self.ui.status_label.setText(self.state_strings["not_can_launch"])
            elif (
                self.rgame.igame
                and (self.rgame.game.supports_cloud_saves or self.rgame.game.supports_mac_cloud_saves)
                and not self.rgame.is_save_up_to_date
            ):
                self.ui.status_label.setText(self.state_strings["save_not_up_to_date"])
            else:
                self.ui.status_label.setText(self.state_strings[self.rgame.state])
        else:
            self.ui.status_label.setText(self.state_strings[self.rgame.state])
        self.ui.status_label.setVisible(bool(self.ui.status_label.text()))

    @Slot()
    def update_buttons(self):
        self.ui.install_btn.setVisible(not self.rgame.is_installed)
        self.ui.install_btn.setEnabled(not self.rgame.is_installed)
        self.ui.launch_btn.setVisible(self.rgame.is_installed)
        self.ui.launch_btn.setEnabled(self.rgame.can_launch)

        self.steam_shortcut_action.setEnabled(self.rgame.has_pixmap)

    @Slot()
    def update_actions(self):
        for action in self.actions():
            self.removeAction(action)

        if self.rgame.is_installed or self.rgame.is_origin:
            self.addAction(self.launch_action)
        else:
            self.addAction(self.install_action)

        if desktop_links_supported() and self.rgame.is_installed:
            if desktop_link_path(self.rgame.folder_name, "desktop").exists():
                self.desktop_link_action.setText(self.tr("Remove Desktop link"))
            else:
                self.desktop_link_action.setText(self.tr("Create Desktop link"))
            self.addAction(self.desktop_link_action)
            if desktop_link_path(self.rgame.folder_name, "start_menu").exists():
                self.menu_link_action.setText(self.tr("Remove Start Menu link"))
            else:
                self.menu_link_action.setText(self.tr("Create Start Menu link"))
            self.addAction(self.menu_link_action)

        if steam_shortcuts_supported() and self.rgame.is_installed:
            if steam_shortcut_exists(self.rgame.app_name):
                self.steam_shortcut_action.setText(self.tr("Remove Steam shortcut"))
            else:
                self.steam_shortcut_action.setText(self.tr("Create Steam shortcut"))
            self.addAction(self.steam_shortcut_action)

        self.addAction(self.reload_action)
        if self.rgame.is_installed and not self.rgame.is_origin:
            self.addAction(self.uninstall_action)

    def eventFilter(self, a0: QObject, a1: QEvent) -> bool:
        if not isinstance(a1, QEvent):
            # FIXME: investigate why this happens
            #
            # ERROR: Supplied arg1 <class 'PySide6.QtCore.QRunnable'> with target \
            # <class 'rare.components.tabs.library.widgets.icon_game_widget.IconGameWidget'> \
            # is not a QEvent object
            logger.error("Supplied arg1 %s with target %s is not a QEvent object", type(a1), type(a0))
            return True
        if a0 is self.ui.launch_btn:
            if a1.type() == QEvent.Type.Enter:
                if not self.rgame.can_launch:
                    self.ui.tooltip_label.setText(self.hover_strings["not_can_launch"])
                elif self.rgame.is_origin:
                    self.ui.tooltip_label.setText(self.hover_strings["is_origin"])
                elif self.rgame.has_update:
                    self.ui.tooltip_label.setText(self.hover_strings["has_update"])
                elif self.rgame.is_foreign and self.rgame.can_run_offline:
                    self.ui.tooltip_label.setText(self.hover_strings["is_foreign"])
                elif self.rgame.can_launch:
                    self.ui.tooltip_label.setText(self.hover_strings["can_launch"])
                return True
            if a1.type() == QEvent.Type.Leave:
                self.ui.tooltip_label.setText(self.hover_strings["info"])
                # return True
        if a0 is self.ui.install_btn:
            if a1.type() == QEvent.Type.Enter:
                self.ui.tooltip_label.setText(self.hover_strings["install"])
                return True
            if a1.type() == QEvent.Type.Leave:
                self.ui.tooltip_label.setText(self.hover_strings["info"])
                # return True
        if a0 is self:
            if a1.type() == QEvent.Type.Enter:
                self.ui.tooltip_label.setText(self.hover_strings["info"])
        return super(GameWidget, self).eventFilter(a0, a1)

    def mousePressEvent(self, e: QMouseEvent) -> None:
        # left button
        if e.button() == Qt.MouseButton.LeftButton:
            self.show_info.emit(self.rgame)
            e.accept()
        # right
        elif e.button() == Qt.MouseButton.RightButton:
            super(GameWidget, self).mousePressEvent(e)

    @Slot()
    def _on_reload_image(self) -> None:
        self.rgame.refresh_pixmap()

    @Slot()
    @Slot(bool, bool)
    def _launch(self, offline=False, skip_version_check=False):
        if offline or (self.rgame.is_foreign and self.rgame.can_run_offline):
            offline = True
        if self.rgame.has_update:
            skip_version_check = True
        self.rgame.launch(offline=offline, skip_update_check=skip_version_check)

    @Slot()
    def _install(self):
        self.show_info.emit(self.rgame)

    @Slot()
    def _uninstall(self):
        self.show_info.emit(self.rgame)

    @Slot(str, str)
    def _create_link(self, name: str, link_type: str):
        if not desktop_links_supported():
            QMessageBox.warning(
                self,
                self.tr("Warning"),
                self.tr("Creating shortcuts is currently unsupported on {}").format(platform.system()),
            )
            return

        shortcut_path = desktop_link_path(name, link_type)

        if not shortcut_path.exists():
            try:
                if not create_desktop_link(
                    app_name=self.rgame.app_name,
                    app_title=self.rgame.app_title,
                    link_name=self.rgame.folder_name,
                    link_type=link_type,
                ):
                    raise PermissionError
            except PermissionError:
                QMessageBox.warning(self, "Error", "Could not create shortcut.")
                return
        else:
            if shortcut_path.exists():
                shortcut_path.unlink(missing_ok=True)
        self.update_actions()

    @Slot(str, str)
    def _create_steam_shortcut(self, app_name: str, app_title: str):
        if steam_shortcut_exists(app_name):
            if shortcut := remove_steam_shortcut(app_name):
                remove_steam_coverart(shortcut)
        else:
            if shortcut := add_steam_shortcut(app_name, app_title):
                add_steam_coverart(app_name, shortcut)
        save_steam_shortcuts()
        self.update_actions()



import platform
import time
from logging import getLogger
from typing import List

from PySide6.QtCore import QObject, QSettings, Slot
from pypresence import Presence, exceptions

from rare.models.options import options
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton

client_id = "830732538225360908"
logger = getLogger("DiscordRPC")


class DiscordRPC(QObject):
    def __init__(self):
        super(DiscordRPC, self).__init__()
        self.rpc = None
        self.state = 1  # 0: game, 1: always active, 2: off
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()

        self.settings = QSettings()
        if self.settings.value(*options.discord_rpc_mode) == 1:  # show always
            self.state = 2
            self.set_discord_rpc()

        self.signals.discord_rpc.update_presence.connect(self.update_presence)
        self.signals.discord_rpc.remove_presence.connect(self.remove_presence)
        self.signals.discord_rpc.update_settings.connect(self.update_settings)

    @Slot(str)
    def update_presence(self, app_name: str):
        self.set_discord_rpc(app_name)

    @Slot(str)
    def remove_presence(self, app_name: str):
        self.set_discord_rpc(None)

    @Slot()
    @Slot(list)
    def update_settings(self, game_running: List = None):
        value = self.settings.value(*options.discord_rpc_mode)
        if value == 2:
            self.remove_rpc()
            return
        if not game_running and value == 0:
            self.remove_rpc()
        elif not game_running:
            self.set_discord_rpc()
        else:
            self.set_discord_rpc(game_running[0])

    def remove_rpc(self):
        if self.settings.value(*options.discord_rpc_mode) != 1:
            if not self.rpc:
                return
            try:
                self.rpc.close()
            except Exception:
                logger.warning("Already closed")
            del self.rpc
            self.rpc = None
            logger.info("Remove RPC")
            self.state = 2
        else:
            self.state = 1
            self.set_discord_rpc()

    def set_discord_rpc(self, app_name=None):
        if not self.rpc:
            try:
                self.rpc = Presence(client_id)  # Rare app: https://discord.com/developers/applications
                self.rpc.connect()
            except ConnectionRefusedError as e:
                logger.warning(f"Discord is not active\n{e}")
                self.rpc = None
                return
            except FileNotFoundError as e:
                logger.warning(f"File not found error\n{e}")
                self.rpc = None
                return
            except exceptions.InvalidPipe as e:
                logger.error(f"Is Discord running? \n{e}")
                self.rpc = None
                return
            except Exception as e:
                logger.error(str(e))
                self.rpc = None
                return
        self.update_rpc(app_name)

    def update_rpc(self, app_name=None):
        if self.settings.value(*options.discord_rpc_mode) == 2 or (
            not app_name and self.settings.value(*options.discord_rpc_mode) == 0
        ):
            self.remove_rpc()
            return
        title = None
        if not app_name:
            self.rpc.update(large_image="logo", details="https://github.com/RareDevs/Rare")
            return
        if self.settings.value(*options.discord_rpc_game):
            try:
                title = self.core.get_installed_game(app_name).title
            except AttributeError:
                logger.error(f"Could not get title of game: {app_name}")
                title = app_name
        start = None
        if self.settings.value(*options.discord_rpc_time):
            start = str(time.time()).split(".")[0]
        os = None
        if self.settings.value(*options.discord_rpc_os):
            os = f"via Rare on {platform.system()}"

        self.rpc.update(large_image="logo", details=title, large_text=title, state=os, start=start)
        self.state = 0



from PySide6.QtCore import Signal, Slot
from PySide6.QtGui import QShortcut
from PySide6.QtWidgets import QMenu, QTabWidget, QWidget, QWidgetAction, QMessageBox

from rare.shared import RareCore, LegendaryCoreSingleton, GlobalSignalsSingleton, ArgumentsSingleton
from rare.utils.misc import qta_icon, ExitCodes
from .account import AccountWidget
from .downloads import DownloadsTab
from .library import GamesLibrary
from .settings import SettingsTab
from .store import StoreTab
from .tab_widgets import MainTabBar, TabButtonWidget


class MainTabWidget(QTabWidget):
    # int: exit code
    exit_app: Signal = Signal(int)

    def __init__(self, parent):
        super(MainTabWidget, self).__init__(parent=parent)

        self.rcore = RareCore.instance()
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()

        self.tab_bar = MainTabBar(parent=self)
        self.setTabBar(self.tab_bar)

        # Generate Tabs
        self.games_tab = GamesLibrary(self)
        self.games_index = self.addTab(self.games_tab, self.tr("Games"))

        # Downloads Tab after Games Tab to use populated RareCore games list
        self.downloads_tab = DownloadsTab(self)
        self.downloads_index = self.addTab(self.downloads_tab, "")
        self.downloads_tab.update_title.connect(self.__on_downloads_update_title)
        self.downloads_tab.update_queues_count()
        self.setTabEnabled(self.downloads_index, not self.args.offline)

        if not self.args.offline:
            self.store_tab = StoreTab(self.core, parent=self)
            self.store_index = self.addTab(self.store_tab, self.tr("Store (Beta)"))
            self.setTabEnabled(self.store_index, not self.args.offline)

        # Space Tab
        space_index = self.addTab(QWidget(self), "")
        self.setTabEnabled(space_index, False)
        self.tab_bar.expanded = space_index

        # Settings Tab
        self.settings_tab = SettingsTab(self)
        self.settings_index = self.addTab(self.settings_tab, qta_icon("fa.gear"), "")
        self.settings_tab.about.update_available_ready.connect(
            lambda: self.tab_bar.setTabText(self.settings_index, "(!)")
        )

        # Account Button
        self.account_widget = AccountWidget(self)
        self.account_widget.exit_app.connect(self.__on_exit_app)
        account_action = QWidgetAction(self)
        account_action.setDefaultWidget(self.account_widget)
        account_button = TabButtonWidget(qta_icon("mdi.account-circle", fallback="fa.user"), tooltip="Menu")
        account_menu = QMenu(account_button)
        account_menu.addAction(account_action)
        account_button.setMenu(account_menu)
        self.tab_bar.setButton(account_button)

        # Open game list on click on Games tab button
        self.tabBarClicked.connect(self.mouse_clicked)

        # shortcuts
        QShortcut("Alt+1", self).activated.connect(lambda: self.setCurrentIndex(self.games_index))
        if not self.args.offline:
            QShortcut("Alt+2", self).activated.connect(lambda: self.setCurrentIndex(self.downloads_index))
            QShortcut("Alt+3", self).activated.connect(lambda: self.setCurrentIndex(self.store_index))
        QShortcut("Alt+4", self).activated.connect(lambda: self.setCurrentIndex(self.settings_index))

    @Slot(int)
    def __on_downloads_update_title(self, num_downloads: int):
        self.setTabText(self.indexOf(self.downloads_tab), self.tr("Downloads ({})").format(num_downloads))

    def mouse_clicked(self, index):
        if index == self.games_index:
            self.games_tab.setCurrentWidget(self.games_tab.games_page)

    def resizeEvent(self, event):
        self.tab_bar.setMinimumWidth(self.width())
        super(MainTabWidget, self).resizeEvent(event)

    @Slot(int)
    def __on_exit_app(self, exit_code: int):
        # FIXME: Don't allow logging out if there are active downloads
        if self.downloads_tab.is_download_active:
            QMessageBox.warning(
                self,
                self.tr("Quit") if exit_code == ExitCodes.EXIT else self.tr("Logout"),
                self.tr("There are active downloads. Stop them before trying to quit."),
            )
            return
        # FIXME: End of FIXME
        if exit_code == ExitCodes.LOGOUT:
            reply = QMessageBox.question(
                self,
                self.tr("Logout"),
                self.tr("Do you really want to logout <b>{}</b>?").format(self.core.lgd.userdata.get("display_name")),
                buttons=(QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No),
                defaultButton=QMessageBox.StandardButton.No,
            )

            if reply == QMessageBox.StandardButton.Yes:
                self.core.lgd.invalidate_userdata()
            else:
                return
        self.exit_app.emit(exit_code)  # restart exit code



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'dlcs.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QLabel, QLayout,
    QSizePolicy, QToolBox, QVBoxLayout, QWidget)

class Ui_GameDlcs(object):
    def setupUi(self, GameDlcs):
        if not GameDlcs.objectName():
            GameDlcs.setObjectName(u"GameDlcs")
        GameDlcs.resize(271, 141)
        GameDlcs.setWindowTitle(u"GameDlcs")
        GameDlcs.setFrameShape(QFrame.Shape.StyledPanel)
        GameDlcs.setFrameShadow(QFrame.Sunken)
        GameDlcs.setLineWidth(0)
        self.installed_dlc_page = QWidget()
        self.installed_dlc_page.setObjectName(u"installed_dlc_page")
        self.installed_dlc_page.setGeometry(QRect(0, 0, 287, 62))
        self.installed_dlc_page_layout = QVBoxLayout(self.installed_dlc_page)
        self.installed_dlc_page_layout.setObjectName(u"installed_dlc_page_layout")
        self.installed_dlc_page_layout.setContentsMargins(0, 0, 0, 0)
        self.installed_dlc_label = QLabel(self.installed_dlc_page)
        self.installed_dlc_label.setObjectName(u"installed_dlc_label")

        self.installed_dlc_page_layout.addWidget(self.installed_dlc_label, 0, Qt.AlignmentFlag.AlignTop)

        self.installed_dlc_container = QWidget(self.installed_dlc_page)
        self.installed_dlc_container.setObjectName(u"installed_dlc_container")
        self.installed_dlc_container_layout = QVBoxLayout(self.installed_dlc_container)
        self.installed_dlc_container_layout.setObjectName(u"installed_dlc_container_layout")
        self.installed_dlc_container_layout.setContentsMargins(0, 0, 3, 0)

        self.installed_dlc_page_layout.addWidget(self.installed_dlc_container, 0, Qt.AlignmentFlag.AlignTop)

        self.installed_dlc_page_layout.setStretch(1, 1)
        GameDlcs.addItem(self.installed_dlc_page, u"Installed DLCs")
        self.available_dlc_page = QWidget()
        self.available_dlc_page.setObjectName(u"available_dlc_page")
        self.available_dlc_page.setGeometry(QRect(0, 0, 271, 83))
        self.available_dlc_page_layou = QVBoxLayout(self.available_dlc_page)
        self.available_dlc_page_layou.setObjectName(u"available_dlc_page_layou")
        self.available_dlc_page_layou.setContentsMargins(0, 0, 0, 0)
        self.available_dlc_label = QLabel(self.available_dlc_page)
        self.available_dlc_label.setObjectName(u"available_dlc_label")

        self.available_dlc_page_layou.addWidget(self.available_dlc_label, 0, Qt.AlignmentFlag.AlignTop)

        self.available_dlc_container = QWidget(self.available_dlc_page)
        self.available_dlc_container.setObjectName(u"available_dlc_container")
        self.available_dlc_container_layout = QVBoxLayout(self.available_dlc_container)
        self.available_dlc_container_layout.setObjectName(u"available_dlc_container_layout")
        self.available_dlc_container_layout.setContentsMargins(0, 0, 3, 0)

        self.available_dlc_page_layou.addWidget(self.available_dlc_container, 0, Qt.AlignmentFlag.AlignTop)

        self.available_dlc_page_layou.setStretch(1, 1)
        GameDlcs.addItem(self.available_dlc_page, u"Available DLCs")

        self.retranslateUi(GameDlcs)
    # setupUi

    def retranslateUi(self, GameDlcs):
        self.installed_dlc_label.setText(QCoreApplication.translate("GameDlcs", u"No Downloadable Content has been installed.", None))
        GameDlcs.setItemText(GameDlcs.indexOf(self.installed_dlc_page), QCoreApplication.translate("GameDlcs", u"Installed DLCs", None))
        self.available_dlc_label.setText(QCoreApplication.translate("GameDlcs", u"No Downloadable Content is available", None))
        GameDlcs.setItemText(GameDlcs.indexOf(self.available_dlc_page), QCoreApplication.translate("GameDlcs", u"Available DLCs", None))
        pass
    # retranslateUi




import logging

from PySide6.QtCore import Signal, Qt
from PySide6.QtGui import QMouseEvent
from PySide6.QtWidgets import QPushButton

from rare.components.tabs.store.api.models.response import CatalogOfferModel
from rare.models.image import ImageSize
from rare.utils.misc import qta_icon
from rare.utils.qt_requests import QtRequests
from .image import LoadingImageWidget

logger = logging.getLogger("StoreWidgets")


class ItemWidget(LoadingImageWidget):
    show_details = Signal(CatalogOfferModel)

    def __init__(self, manager: QtRequests, catalog_game: CatalogOfferModel = None, parent=None):
        super(ItemWidget, self).__init__(manager, parent=parent)
        self.catalog_game = catalog_game

    def mousePressEvent(self, a0: QMouseEvent) -> None:
        if a0.button() == Qt.MouseButton.LeftButton:
            a0.accept()
            self.show_details.emit(self.catalog_game)
        if a0.button() == Qt.MouseButton.RightButton:
            a0.accept()


class StoreItemWidget(ItemWidget):
    def __init__(self, manager: QtRequests, catalog_game: CatalogOfferModel = None, parent=None):
        super(StoreItemWidget, self).__init__(manager, catalog_game, parent=parent)
        self.setFixedSize(ImageSize.DisplayWide)
        self.ui.setupUi(self)
        if catalog_game:
            self.init_ui(catalog_game)

    def init_ui(self, game: CatalogOfferModel):
        if not game:
            self.ui.title_label.setText(self.tr("An error occurred"))
            return

        self.ui.title_label.setText(game.title)
        for attr in game.customAttributes:
            if attr["key"] == "developerName":
                developer = attr["value"]
                break
        else:
            developer = game.seller["name"]
        self.ui.developer_label.setText(developer)
        price = game.price.totalPrice.fmtPrice["originalPrice"]
        discount_price = game.price.totalPrice.fmtPrice["discountPrice"]
        self.ui.price_label.setText(f'{price if price != "0" else self.tr("Free")}')
        if price != discount_price:
            font = self.ui.price_label.font()
            font.setStrikeOut(True)
            self.ui.price_label.setFont(font)
            self.ui.discount_label.setText(f'{discount_price if discount_price != "0" else self.tr("Free")}')
        else:
            self.ui.discount_label.setVisible(False)

        key_images = game.keyImages
        self.fetchPixmap(key_images.for_dimensions(self.width(), self.height()).url)

        # for img in json_info["keyImages"]:
        #     if img["type"] in ["DieselStoreFrontWide", "OfferImageWide", "VaultClosed", "ProductLogo"]:
        #         if img["type"] == "VaultClosed" and json_info["title"] != "Mystery Game":
        #             continue
        #         self.fetchPixmap(img["url"])
        #         break
        # else:
        #     logger.info(", ".join([img["type"] for img in json_info["keyImages"]]))


class SearchItemWidget(ItemWidget):
    def __init__(self, manager: QtRequests, catalog_game: CatalogOfferModel, parent=None):
        super(SearchItemWidget, self).__init__(manager, catalog_game, parent=parent)
        self.setFixedSize(ImageSize.LibraryTall)
        self.ui.setupUi(self)

        key_images = catalog_game.keyImages
        self.fetchPixmap(key_images.for_dimensions(self.width(), self.height()).url)

        self.ui.title_label.setText(catalog_game.title)

        price = catalog_game.price.totalPrice.fmtPrice["originalPrice"]
        discount_price = catalog_game.price.totalPrice.fmtPrice["discountPrice"]
        self.ui.price_label.setText(f'{price if price != "0" else self.tr("Free")}')
        if price != discount_price:
            font = self.ui.price_label.font()
            font.setStrikeOut(True)
            self.ui.price_label.setFont(font)
            self.ui.discount_label.setText(f'{discount_price if discount_price != "0" else self.tr("Free")}')
        else:
            self.ui.discount_label.setVisible(False)


class WishlistItemWidget(ItemWidget):
    delete_from_wishlist = Signal(CatalogOfferModel)

    def __init__(self, manager: QtRequests, catalog_game: CatalogOfferModel, parent=None):
        super(WishlistItemWidget, self).__init__(manager, catalog_game, parent=parent)
        self.setFixedSize(ImageSize.DisplayWide)
        self.ui.setupUi(self)
        for attr in catalog_game.customAttributes:
            if attr["key"] == "developerName":
                developer = attr["value"]
                break
        else:
            developer = catalog_game.seller["name"]
        original_price = catalog_game.price.totalPrice.fmtPrice["originalPrice"]
        discount_price = catalog_game.price.totalPrice.fmtPrice["discountPrice"]

        self.ui.title_label.setText(catalog_game.title)
        self.ui.developer_label.setText(developer)
        self.ui.price_label.setText(f'{original_price if original_price != "0" else self.tr("Free")}')
        if original_price != discount_price:
            font = self.ui.price_label.font()
            font.setStrikeOut(True)
            self.ui.price_label.setFont(font)
            self.ui.discount_label.setText(f'{discount_price if discount_price != "0" else self.tr("Free")}')
        else:
            self.ui.discount_label.setVisible(False)
        key_images = catalog_game.keyImages
        self.fetchPixmap(
            key_images.for_dimensions(self.width(), self.height()).url
        )

        self.delete_button = QPushButton(self)
        self.delete_button.setIcon(qta_icon("mdi.delete", color="white"))
        self.delete_button.clicked.connect(
            lambda: self.delete_from_wishlist.emit(self.catalog_game)
        )
        self.layout().insertWidget(0, self.delete_button, alignment=Qt.AlignmentFlag.AlignRight)




import os
import shlex
from hashlib import md5
from enum import IntEnum
from typing import Dict, List, Union


class WrapperType(IntEnum):
    NONE = 0
    COMPAT_TOOL = 1
    LEGENDARY_IMPORT = 8
    USER_DEFINED = 9


class Wrapper:
    def __init__(self, command: Union[str, List[str]], name: str = None, wtype: WrapperType = None):
        self.__command: List[str] = shlex.split(command) if isinstance(command, str) else command
        self.__name: str = name if name is not None else os.path.basename(self.__command[0])
        self.__wtype: WrapperType = wtype if wtype is not None else WrapperType.USER_DEFINED

    @property
    def is_compat_tool(self) -> bool:
        return self.__wtype == WrapperType.COMPAT_TOOL

    @property
    def is_editable(self) -> bool:
        return self.__wtype in {WrapperType.USER_DEFINED, WrapperType.LEGENDARY_IMPORT}

    @property
    def checksum(self) -> str:
        return md5(self.as_str.encode("utf-8")).hexdigest()

    @property
    def executable(self) -> str:
        return shlex.quote(self.__command[0])

    @property
    def command(self) -> List[str]:
        return self.__command

    @property
    def as_str(self) -> str:
        # return " ".join(shlex.quote(part) for part in self.__command)
        return " ".join(map(shlex.quote, self.__command))

    @property
    def name(self) -> str:
        return self.__name

    @property
    def type(self) -> WrapperType:
        return self.__wtype

    def __eq__(self, other) -> bool:
        return self.as_str == other.as_str

    def __hash__(self):
        return hash(self.__command)

    def __bool__(self) -> bool:
        return True if not self.is_editable else bool(self.as_str.strip())

    @classmethod
    def from_dict(cls, data: Dict):
        return cls(
            command=data.get("command"),
            name=data.get("name"),
            wtype=WrapperType(data.get("wtype", WrapperType.USER_DEFINED))
        )

    @property
    def __dict__(self):
        return dict(
            command=self.__command,
            name=self.__name,
            wtype=int(self.__wtype)
        )



import webbrowser
from logging import getLogger
from typing import Tuple

from PySide6.QtCore import Signal
from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import QWidget

from rare import __version__, __codename__
from rare.ui.components.tabs.settings.about import Ui_About
from rare.utils.qt_requests import QtRequests

logger = getLogger("About")


def versiontuple(v) -> Tuple[int, ...]:
    try:
        return tuple(map(int, (v.split("."))))
    except Exception as e:
        logger.error("Error while parsing version %s", v)
        logger.error(e)
        return 99, 99, 99, 999


class About(QWidget):
    update_available_ready = Signal()

    def __init__(self, parent=None):
        super(About, self).__init__(parent=parent)
        self.ui = Ui_About()
        self.ui.setupUi(self)

        self.ui.version.setText(f"{__version__}  {__codename__}")

        self.ui.update_label.setEnabled(False)
        self.ui.update_lbl.setEnabled(False)
        self.ui.open_browser.setVisible(False)
        self.ui.open_browser.setEnabled(False)

        self.releases_url = "https://api.github.com/repos/RareDevs/Rare/releases/latest"

        self.manager = QtRequests(parent=self)
        self.manager.get(self.releases_url, self.update_available_finished)

        self.ui.open_browser.clicked.connect(
            lambda: webbrowser.open("https://github.com/RareDevs/Rare/releases/latest")
        )

        self.update_available = False

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.manager.get(self.releases_url, self.update_available_finished)
        super().showEvent(a0)

    def update_available_finished(self, data: dict):
        if latest_tag := data.get("tag_name"):
            self.update_available = versiontuple(latest_tag) > versiontuple(__version__)
        else:
            self.update_available = False

        if self.update_available:
            logger.info(f"Update available: {__version__} -> {latest_tag}")
            self.ui.update_lbl.setText(f"{__version__} -> {latest_tag}")
            self.update_available_ready.emit()
        else:
            self.ui.update_lbl.setText(self.tr("You have the latest version"))
        self.ui.update_label.setEnabled(self.update_available)
        self.ui.update_lbl.setEnabled(self.update_available)
        self.ui.open_browser.setVisible(self.update_available)
        self.ui.open_browser.setEnabled(self.update_available)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'queue_info_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QLabel, QLayout,
    QSizePolicy, QSpacerItem, QVBoxLayout, QWidget)

class Ui_QueueInfoWidget(object):
    def setupUi(self, QueueInfoWidget):
        if not QueueInfoWidget.objectName():
            QueueInfoWidget.setObjectName(u"QueueInfoWidget")
        QueueInfoWidget.resize(624, 60)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(QueueInfoWidget.sizePolicy().hasHeightForWidth())
        QueueInfoWidget.setSizePolicy(sizePolicy)
        QueueInfoWidget.setWindowTitle(u"QueueInfoWidget")
        self.queue_info_layout = QGridLayout(QueueInfoWidget)
        self.queue_info_layout.setObjectName(u"queue_info_layout")
        self.queue_info_layout.setContentsMargins(0, 0, 0, 0)
        self.dl_size_label = QLabel(QueueInfoWidget)
        self.dl_size_label.setObjectName(u"dl_size_label")
        font = QFont()
        font.setBold(True)
        self.dl_size_label.setFont(font)
        self.dl_size_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.queue_info_layout.addWidget(self.dl_size_label, 1, 3, 1, 1)

        self.title = QLabel(QueueInfoWidget)
        self.title.setObjectName(u"title")
        self.title.setFont(font)
        self.title.setText(u"title")

        self.queue_info_layout.addWidget(self.title, 0, 1, 1, 4)

        self.local_version_label = QLabel(QueueInfoWidget)
        self.local_version_label.setObjectName(u"local_version_label")
        self.local_version_label.setFont(font)
        self.local_version_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.queue_info_layout.addWidget(self.local_version_label, 2, 1, 1, 1)

        self.image_layout = QVBoxLayout()
        self.image_layout.setObjectName(u"image_layout")
        self.image_layout.setSizeConstraint(QLayout.SetFixedSize)

        self.queue_info_layout.addLayout(self.image_layout, 0, 0, 3, 1)

        self.install_size_label = QLabel(QueueInfoWidget)
        self.install_size_label.setObjectName(u"install_size_label")
        self.install_size_label.setFont(font)
        self.install_size_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.queue_info_layout.addWidget(self.install_size_label, 2, 3, 1, 1)

        self.install_size = QLabel(QueueInfoWidget)
        self.install_size.setObjectName(u"install_size")
        self.install_size.setMinimumSize(QSize(160, 0))
        self.install_size.setText(u"install_size")

        self.queue_info_layout.addWidget(self.install_size, 2, 4, 1, 1)

        self.info_hspacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.queue_info_layout.addItem(self.info_hspacer, 0, 5, 3, 1)

        self.dl_size = QLabel(QueueInfoWidget)
        self.dl_size.setObjectName(u"dl_size")
        self.dl_size.setMinimumSize(QSize(160, 0))
        self.dl_size.setText(u"dl_size")

        self.queue_info_layout.addWidget(self.dl_size, 1, 4, 1, 1)

        self.remote_version_label = QLabel(QueueInfoWidget)
        self.remote_version_label.setObjectName(u"remote_version_label")
        self.remote_version_label.setFont(font)
        self.remote_version_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.queue_info_layout.addWidget(self.remote_version_label, 1, 1, 1, 1)

        self.local_version = QLabel(QueueInfoWidget)
        self.local_version.setObjectName(u"local_version")
        self.local_version.setMinimumSize(QSize(160, 0))
        self.local_version.setMaximumSize(QSize(160, 16777215))
        self.local_version.setText(u"local_version")

        self.queue_info_layout.addWidget(self.local_version, 2, 2, 1, 1)

        self.remote_version = QLabel(QueueInfoWidget)
        self.remote_version.setObjectName(u"remote_version")
        self.remote_version.setMinimumSize(QSize(160, 0))
        self.remote_version.setMaximumSize(QSize(160, 16777215))
        self.remote_version.setText(u"remote_version")

        self.queue_info_layout.addWidget(self.remote_version, 1, 2, 1, 1)


        self.retranslateUi(QueueInfoWidget)
    # setupUi

    def retranslateUi(self, QueueInfoWidget):
        self.dl_size_label.setText(QCoreApplication.translate("QueueInfoWidget", u"Download size", None))
        self.local_version_label.setText(QCoreApplication.translate("QueueInfoWidget", u"Local version", None))
        self.install_size_label.setText(QCoreApplication.translate("QueueInfoWidget", u"Installed size", None))
        self.remote_version_label.setText(QCoreApplication.translate("QueueInfoWidget", u"Remote version", None))
        pass
    # retranslateUi




import platform as pf
from logging import getLogger

from .widgets.env_vars import EnvVars
from .widgets.game import GameSettingsBase
from .widgets.launch import LaunchSettingsBase
from .widgets.overlay import DxvkSettings
from .widgets.wrappers import WrapperSettings

if pf.system() != "Windows":
    from .widgets.wine import WineSettings
    if pf.system() in {"Linux", "FreeBSD"}:
        from .widgets.proton import ProtonSettings
        from .widgets.overlay import MangoHudSettings

logger = getLogger("GameSettings")


class LaunchSettings(LaunchSettingsBase):
    def __init__(self, parent=None):
        super(LaunchSettings, self).__init__(WrapperSettings, parent=parent)


class GameSettings(GameSettingsBase):
    def __init__(self, parent=None):
        if pf.system() != "Windows":
            if pf.system() in {"Linux", "FreeBSD"}:
                super(GameSettings, self).__init__(
                    LaunchSettings, DxvkSettings, EnvVars,
                    WineSettings, ProtonSettings, MangoHudSettings,
                    parent=parent
                )
            else:
                super(GameSettings, self).__init__(
                    LaunchSettings, DxvkSettings, EnvVars,
                    WineSettings,
                    parent=parent
                )
        else:
            super(GameSettings, self).__init__(
                LaunchSettings, DxvkSettings, EnvVars,
                parent=parent
            )



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'console_env.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QAbstractItemView, QApplication, QDialog,
    QDialogButtonBox, QHeaderView, QSizePolicy, QTableWidget,
    QTableWidgetItem, QVBoxLayout, QWidget)

class Ui_ConsoleEnv(object):
    def setupUi(self, ConsoleEnv):
        if not ConsoleEnv.objectName():
            ConsoleEnv.setObjectName(u"ConsoleEnv")
        ConsoleEnv.resize(600, 400)
        ConsoleEnv.setWindowTitle(u"ConsoleEnv")
        self.layout = QVBoxLayout(ConsoleEnv)
        self.layout.setObjectName(u"layout")
        self.table = QTableWidget(ConsoleEnv)
        if (self.table.columnCount() < 2):
            self.table.setColumnCount(2)
        __qtablewidgetitem = QTableWidgetItem()
        self.table.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.table.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        self.table.setObjectName(u"table")
        font = QFont()
        font.setFamilies([u"Monospace"])
        self.table.setFont(font)
        self.table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.table.setAlternatingRowColors(True)
        self.table.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.table.setSortingEnabled(True)
        self.table.setCornerButtonEnabled(True)
        self.table.setColumnCount(2)
        self.table.horizontalHeader().setVisible(True)
        self.table.horizontalHeader().setStretchLastSection(True)
        self.table.verticalHeader().setVisible(False)

        self.layout.addWidget(self.table)

        self.buttons = QDialogButtonBox(ConsoleEnv)
        self.buttons.setObjectName(u"buttons")
        self.buttons.setOrientation(Qt.Orientation.Horizontal)
        self.buttons.setStandardButtons(QDialogButtonBox.Close)

        self.layout.addWidget(self.buttons)


        self.retranslateUi(ConsoleEnv)
        self.buttons.accepted.connect(ConsoleEnv.accept)
        self.buttons.rejected.connect(ConsoleEnv.reject)
    # setupUi

    def retranslateUi(self, ConsoleEnv):
        ___qtablewidgetitem = self.table.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("ConsoleEnv", u"Variable", None));
        ___qtablewidgetitem1 = self.table.horizontalHeaderItem(1)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("ConsoleEnv", u"Value", None));
        pass
    # retranslateUi




from abc import abstractmethod
from typing import Tuple, List, Union, Type, TypeVar

from PySide6.QtCore import QObject, Slot, Qt
from PySide6.QtWidgets import QWidget, QVBoxLayout, QScrollArea

from rare.lgndr.core import LegendaryCore
from rare.models.game import RareGame
from rare.models.signals import GlobalSignals
from rare.models.library import LibraryFilter, LibraryOrder, LibraryView
from rare.shared import RareCore
from rare.widgets.library_layout import LibraryLayout
from .icon_game_widget import IconGameWidget
from .list_game_widget import ListGameWidget

ViewWidget = TypeVar("ViewWidget", IconGameWidget, ListGameWidget)


class ViewContainer(QWidget):
    def __init__(self, rcore: RareCore, parent=None):
        super().__init__(parent=parent)
        self.rcore: RareCore = rcore

    def _add_widget(self, widget_type: Type[ViewWidget], rgame: RareGame) -> ViewWidget:
        widget = widget_type(rgame, self)
        self.layout().addWidget(widget)
        return widget

    @staticmethod
    def __visibility(widget: ViewWidget, library_filter, search_text) -> Tuple[bool, float]:
        if library_filter == LibraryFilter.HIDDEN:
            visible = "hidden" in widget.rgame.metadata.tags
        elif "hidden" in widget.rgame.metadata.tags:
            visible = False
        elif library_filter == LibraryFilter.INSTALLED:
            visible = widget.rgame.is_installed and not widget.rgame.is_unreal
        elif library_filter == LibraryFilter.OFFLINE:
            visible = widget.rgame.can_run_offline and not widget.rgame.is_unreal
        elif library_filter == LibraryFilter.WIN32:
            visible = widget.rgame.is_win32 and not widget.rgame.is_unreal
        elif library_filter == LibraryFilter.MAC:
            visible = widget.rgame.is_mac and not widget.rgame.is_unreal
        elif library_filter == LibraryFilter.INSTALLABLE:
            visible = not widget.rgame.is_non_asset and not widget.rgame.is_unreal
        elif library_filter == LibraryFilter.INCLUDE_UE:
            visible = True
        elif library_filter == LibraryFilter.ALL:
            visible = not widget.rgame.is_unreal
        else:
            visible = True

        if (
            search_text not in widget.rgame.app_name.lower()
            and search_text not in widget.rgame.app_title.lower()
        ):
            opacity = 0.25
        else:
            opacity = 1.0

        return visible, opacity

    def _filter_view(self, widget_type: Type[ViewWidget], filter_by: LibraryFilter = LibraryFilter.ALL, search_text: str = ""):
        widgets = self.findChildren(widget_type)
        for iw in widgets:
            visibility, opacity = self.__visibility(iw, filter_by, search_text)
            iw.setOpacity(opacity)
            iw.setVisible(visibility)

    def _update_view(self, widget_type: Type[ViewWidget]):
        widgets = self.findChildren(widget_type)
        app_names = {iw.rgame.app_name for iw in widgets}
        games = list(self.rcore.games)
        game_app_names = {g.app_name for g in games}
        new_app_names = game_app_names.difference(app_names)
        for app_name in new_app_names:
            game = self.rcore.get_game(app_name)
            w = widget_type(game, self)
            self.layout().addWidget(w)

    def _find_widget(self, widget_type: Type[ViewWidget], app_name: str) -> ViewWidget:
        w = self.findChild(widget_type, app_name)
        return w

    @abstractmethod
    def order_view(self):
        pass


class IconViewContainer(ViewContainer):
    def __init__(self, rcore: RareCore, parent=None):
        super().__init__(rcore, parent=parent)
        view_layout = LibraryLayout(self)
        view_layout.setSpacing(9)
        view_layout.setContentsMargins(0, 13, 0, 13)
        view_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        self.setLayout(view_layout)

    def add_widget(self, rgame: RareGame) -> IconGameWidget:
        return self._add_widget(IconGameWidget, rgame)

    def filter_view(self, filter_by: LibraryFilter = LibraryFilter.ALL, search_text: str = ""):
        self._filter_view(IconGameWidget, filter_by, search_text)

    def update_view(self):
        self._update_view(IconGameWidget)

    def find_widget(self, app_name: str) -> ViewWidget:
        return self._find_widget(IconGameWidget, app_name)

    def order_view(self, order_by: LibraryOrder = LibraryOrder.TITLE, search_text: str = ""):
        if search_text:
            self.layout().sort(
                lambda x: (search_text not in x.widget().rgame.app_title.lower(),)
            )
        else:
            if (newest := order_by == LibraryOrder.NEWEST) or order_by == LibraryOrder.OLDEST:
                # Sort by grant date
                self.layout().sort(
                    key=lambda x: (x.widget().rgame.is_installed, not x.widget().rgame.is_non_asset, x.widget().rgame.grant_date()),
                    reverse=newest,
                )
            elif order_by == LibraryOrder.RECENT:
                # Sort by recently played
                self.layout().sort(
                    key=lambda x: (x.widget().rgame.is_installed, not x.widget().rgame.is_non_asset, x.widget().rgame.metadata.last_played),
                    reverse=True,
                )
            else:
                # Sort by title
                self.layout().sort(
                    key=lambda x: (not x.widget().rgame.is_installed, x.widget().rgame.is_non_asset, x.widget().rgame.app_title)
                )


class ListViewContainer(ViewContainer):
    def __init__(self, rcore, parent=None):
        super().__init__(rcore, parent=parent)
        view_layout = QVBoxLayout(self)
        view_layout.setContentsMargins(3, 3, 9, 3)
        view_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        self.setLayout(view_layout)

    def add_widget(self, rgame: RareGame) -> ListGameWidget:
        return self._add_widget(ListGameWidget, rgame)

    def filter_view(self, filter_by: LibraryFilter = LibraryFilter.ALL, search_text: str = ""):
        self._filter_view(ListGameWidget, filter_by, search_text)

    def update_view(self):
        self._update_view(ListGameWidget)

    def find_widget(self, app_name: str) -> ViewWidget:
        return self._find_widget(ListGameWidget, app_name)

    def order_view(self, order_by: LibraryOrder = LibraryOrder.TITLE, search_text: str = ""):
        list_widgets = self.findChildren(ListGameWidget)
        if search_text:
            list_widgets.sort(key=lambda x: (search_text not in x.rgame.app_title.lower(),))
        else:
            if (newest := order_by == LibraryOrder.NEWEST) or order_by == LibraryOrder.OLDEST:
                list_widgets.sort(
                    key=lambda x: (x.rgame.is_installed, not x.rgame.is_non_asset, x.rgame.grant_date()),
                    reverse=newest,
                )
            elif order_by == LibraryOrder.RECENT:
                list_widgets.sort(
                    key=lambda x: (x.rgame.is_installed, not x.rgame.is_non_asset, x.rgame.metadata.last_played),
                    reverse=True,
                )
            else:
                list_widgets.sort(
                    key=lambda x: (not x.rgame.is_installed, x.rgame.is_non_asset, x.rgame.app_title)
                )
        for idx, wl in enumerate(list_widgets):
            self.layout().insertWidget(idx, wl)


class LibraryWidgetController(QObject):
    def __init__(self, view: LibraryView, parent: QScrollArea = None):
        super(LibraryWidgetController, self).__init__(parent=parent)
        self.rcore = RareCore.instance()
        self.core: LegendaryCore = self.rcore.core()
        self.signals: GlobalSignals = self.rcore.signals()

        if view == LibraryView.COVER:
            self._container: IconViewContainer = IconViewContainer(self.rcore, parent)
        else:
            self._container: ListViewContainer = ListViewContainer(self.rcore, parent)
        parent.setWidget(self._container)

        self.signals.game.installed.connect(self.order_game_views)
        self.signals.game.uninstalled.connect(self.order_game_views)

    def add_game(self, rgame: RareGame):
        return self.add_widgets(rgame)

    def add_widgets(self, rgame: RareGame) -> ViewWidget:
        return self._container.add_widget(rgame)

    def filter_game_views(self, filter_by: LibraryFilter = LibraryFilter.ALL, search_text: str = ""):
        self._container.filter_view(filter_by, search_text)
        self.order_game_views(search_text=search_text)

    @Slot()
    def order_game_views(self, order_by: LibraryOrder = LibraryOrder.TITLE, search_text: str = ""):
        self._container.order_view(order_by, search_text)

    @Slot()
    @Slot(list)
    def update_game_views(self, app_names: List[str] = None):
        if app_names:
            return
        self._container.update_view()
        self.order_game_views()

    def __find_widget(self, app_name: str) -> Union[ViewWidget, None]:
        return self._container.find_widget(app_name)



import logging
from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Dict, Any, Type, Tuple

from .utils import parse_date

logger = logging.getLogger("StoreApiModels")

# lk: Typing overloads for unimplemented types
DieselSocialLinks = Dict

CatalogNamespaceModel = Dict
CategoryModel = Dict
CustomAttributeModel = Dict
ItemModel = Dict
SellerModel = Dict
PageSandboxModel = Dict
TagModel = Dict


@dataclass
class ImageUrlModel:
    type: str = None
    url: str = None

    def as_dict(self) -> Dict[str, Any]:
        tmp: Dict[str, Any] = {}
        tmp.update({})
        if self.type is not None:
            tmp["type"] = self.type
        if self.url is not None:
            tmp["url"] = self.url
        return tmp

    @classmethod
    def from_dict(cls: Type["ImageUrlModel"], src: Dict[str, Any]) -> "ImageUrlModel":
        d = src.copy()
        return cls(type=d.pop("type", ""), url=d.pop("url", ""))


@dataclass
class KeyImagesModel:
    key_images: Tuple[ImageUrlModel, ...] = None
    tall_types = (
        "DieselGameBoxTall",
        "DieselStoreFrontTall",
        "OfferImageTall",
        "DieselGameBoxLogo",
        "Thumbnail",
        "ProductLogo",
    )
    wide_types = (
        "DieselGameBoxwide",
        "DieselStoreFrontWide",
        "OfferImageWide",
        "DieselGameBox",
        "VaultClosed",
        "ProductLogo",
    )

    def __getitem__(self, item):
        return self.key_images[item]

    def __bool__(self):
        return bool(self.key_images)

    @classmethod
    def from_list(cls: Type["KeyImagesModel"], src: List[Dict]):
        d = src.copy()
        key_images = tuple(map(ImageUrlModel.from_dict, d))
        return cls(key_images=key_images)

    def available_tall(self) -> List[ImageUrlModel]:
        tall_images = filter(lambda img: img.type in KeyImagesModel.tall_types, self.key_images)
        tall_images = sorted(tall_images, key=lambda x: KeyImagesModel.tall_types.index(x.type))
        return tall_images

    def available_wide(self) -> List[ImageUrlModel]:
        wide_images = filter(lambda img: img.type in KeyImagesModel.wide_types, self.key_images)
        wide_images = sorted(wide_images, key=lambda x: KeyImagesModel.wide_types.index(x.type))
        return wide_images

    def for_dimensions(self, w: int, h: int) -> ImageUrlModel:
        try:
            if w > h:
                model = self.available_wide()[0]
            else:
                model = self.available_tall()[0]
            _ = model.url
        except Exception as e:
            logger.error(e)
            logger.error(tuple(map(lambda x: x.as_dict(), self.key_images)))
        else:
            return model


CurrencyModel = Dict
FormattedPriceModel = Dict
LineOffersModel = Dict


@dataclass
class TotalPriceModel:
    discountPrice: int = None
    originalPrice: int = None
    voucherDiscount: int = None
    discount: int = None
    currencyCode: str = None
    currencyInfo: CurrencyModel = None
    fmtPrice: FormattedPriceModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["TotalPriceModel"], src: Dict[str, Any]) -> "TotalPriceModel":
        d = src.copy()
        return cls(
            discountPrice=d.pop("discountPrice", 0),
            originalPrice=d.pop("originalPrice", 0),
            voucherDiscount=d.pop("voucherDiscount", 0),
            discount=d.pop("discount", 0),
            currencyCode=d.pop("currencyCode", ""),
            currencyInfo=d.pop("currrencyInfo", {}),
            fmtPrice=d.pop("fmtPrice", {}),
            unmapped=d,
        )


@dataclass
class GetPriceResModel:
    totalPrice: TotalPriceModel = None
    lineOffers: LineOffersModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["GetPriceResModel"], src: Dict[str, Any]) -> "GetPriceResModel":
        d = src.copy()
        total_price = TotalPriceModel.from_dict(x) if (x := d.pop("totalPrice", {})) else None
        return cls(totalPrice=total_price, lineOffers=d.pop("lineOffers", {}), unmapped=d)


DiscountSettingModel = Dict


@dataclass
class PromotionalOfferModel:
    startDate: datetime = None
    endDate: datetime = None
    discountSetting: DiscountSettingModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["PromotionalOfferModel"], src: Dict[str, Any]) -> "PromotionalOfferModel":
        d = src.copy()
        start_date = parse_date(x) if (x := d.pop("startDate", "")) else None
        end_date = parse_date(x) if (x := d.pop("endDate", "")) else None
        return cls(
            startDate=start_date,
            endDate=end_date,
            discountSetting=d.pop("discountSetting", {}),
            unmapped=d,
        )


@dataclass
class PromotionalOffersModel:
    promotionalOffers: Tuple[PromotionalOfferModel, ...] = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_list(cls: Type["PromotionalOffersModel"], src: Dict[str, List]) -> "PromotionalOffersModel":
        d = src.copy()
        promotional_offers = tuple(map(PromotionalOfferModel.from_dict, d.pop("promotionalOffers", [])))
        return cls(promotionalOffers=promotional_offers, unmapped=d)


@dataclass
class PromotionsModel:
    promotionalOffers: Tuple[PromotionalOffersModel, ...] = None
    upcomingPromotionalOffers: Tuple[PromotionalOffersModel, ...] = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["PromotionsModel"], src: Dict[str, Any]) -> "PromotionsModel":
        d = src.copy()
        promotional_offers = tuple(map(PromotionalOffersModel.from_list, d.pop("promotionalOffers", [])))
        upcoming_promotional_offers = tuple(
            map(PromotionalOffersModel.from_list, d.pop("upcomingPromotionalOffers", []))
        )
        return cls(
            promotionalOffers=promotional_offers, upcomingPromotionalOffers=upcoming_promotional_offers, unmapped=d
        )


@dataclass
class CatalogOfferModel:
    catalogNs: CatalogNamespaceModel = None
    categories: List[CategoryModel] = None
    customAttributes: List[CustomAttributeModel] = None
    description: str = None
    effectiveDate: datetime = None
    expiryDate: datetime = None
    id: str = None
    isCodeRedemptionOnly: bool = None
    items: List[ItemModel] = None
    keyImages: KeyImagesModel = None
    namespace: str = None
    offerMappings: List[PageSandboxModel] = None
    offerType: str = None
    price: GetPriceResModel = None
    productSlug: str = None
    promotions: PromotionsModel = None
    seller: SellerModel = None
    status: str = None
    tags: List[TagModel] = None
    title: str = None
    url: str = None
    urlSlug: str = None
    viewableDate: datetime = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["CatalogOfferModel"], src: Dict[str, Any]) -> "CatalogOfferModel":
        d = src.copy()
        effective_date = parse_date(x) if (x := d.pop("effectiveDate", "")) else None
        expiry_date = parse_date(x) if (x := d.pop("expiryDate", "")) else None
        key_images = KeyImagesModel.from_list(d.pop("keyImages", []))
        price = GetPriceResModel.from_dict(x) if (x := d.pop("price", {})) else None
        promotions = PromotionsModel.from_dict(x) if (x := d.pop("promotions", {})) else None
        viewable_date = parse_date(x) if (x := d.pop("viewableDate", "")) else None
        return cls(
            catalogNs=d.pop("catalogNs", {}),
            categories=d.pop("categories", []),
            customAttributes=d.pop("customAttributes", []),
            description=d.pop("description", ""),
            effectiveDate=effective_date,
            expiryDate=expiry_date,
            id=d.pop("id", ""),
            isCodeRedemptionOnly=d.pop("isCodeRedemptionOnly", False),
            items=d.pop("items", []),
            keyImages=key_images,
            namespace=d.pop("namespace", ""),
            offerMappings=d.pop("offerMappings", []),
            offerType=d.pop("offerType", ""),
            price=price,
            productSlug=d.pop("productSlug", ""),
            promotions=promotions,
            seller=d.pop("seller", {}),
            status=d.pop("status", ""),
            tags=d.pop("tags", []),
            title=d.pop("title", ""),
            url=d.pop("url", ""),
            urlSlug=d.pop("urlSlug", ""),
            viewableDate=viewable_date,
            unmapped=d,
        )


@dataclass
class WishlistItemModel:
    created: datetime = None
    id: str = None
    namespace: str = None
    isFirstTime: bool = None
    offerId: str = None
    order: Any = None
    updated: datetime = None
    offer: CatalogOfferModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["WishlistItemModel"], src: Dict[str, Any]) -> "WishlistItemModel":
        d = src.copy()
        created = parse_date(x) if (x := d.pop("created", "")) else None
        offer = CatalogOfferModel.from_dict(x) if (x := d.pop("offer", {})) else None
        updated = parse_date(x) if (x := d.pop("updated", "")) else None
        return cls(
            created=created,
            id=d.pop("id", ""),
            namespace=d.pop("namespace", ""),
            isFirstTime=d.pop("isFirstTime", False),
            offerId=d.pop("offerId", ""),
            order=d.pop("order", ""),
            updated=updated,
            offer=offer,
            unmapped=d,
        )


@dataclass
class PagingModel:
    count: int = None
    total: int = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["PagingModel"], src: Dict[str, Any]) -> "PagingModel":
        d = src.copy()
        count = d.pop("count", 0)
        total = d.pop("total", 0)
        return cls(count=count, total=total, unmapped=d)


@dataclass
class SearchStoreModel:
    elements: Tuple[CatalogOfferModel, ...] = None
    paging: PagingModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["SearchStoreModel"], src: Dict[str, Any]) -> "SearchStoreModel":
        d = src.copy()
        _elements = d.pop("elements", [])
        elements = tuple(map(CatalogOfferModel.from_dict, _elements))
        paging = PagingModel.from_dict(x) if (x := d.pop("paging", {})) else None
        return cls(elements=elements, paging=paging, unmapped=d)


@dataclass
class CatalogModel:
    searchStore: SearchStoreModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["CatalogModel"], src: Dict[str, Any]) -> "CatalogModel":
        d = src.copy()
        search_store = SearchStoreModel.from_dict(x) if (x := d.pop("searchStore", {})) else None
        return cls(searchStore=search_store, unmapped=d)


@dataclass
class WishlistItemsModel:
    elements: Tuple[WishlistItemModel, ...] = None
    paging: PagingModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["WishlistItemsModel"], src: Dict[str, Any]) -> "WishlistItemsModel":
        d = src.copy()
        _elements = d.pop("elements", [])
        elements = tuple(map(WishlistItemModel.from_dict, _elements))
        paging = PagingModel.from_dict(x) if (x := d.pop("paging", {})) else None
        return cls(elements=elements, paging=paging, unmapped=d)


@dataclass
class RemoveFromWishlistModel:
    success: bool = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["RemoveFromWishlistModel"], src: Dict[str, Any]) -> "RemoveFromWishlistModel":
        d = src.copy()
        return cls(success=d.pop("success", False), unmapped=d)


@dataclass
class AddToWishlistModel:
    wishlistItem: WishlistItemModel = None
    success: bool = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["AddToWishlistModel"], src: Dict[str, Any]) -> "AddToWishlistModel":
        d = src.copy()
        wishlist_item = WishlistItemModel.from_dict(x) if (x := d.pop("wishlistItem", {})) else None
        return cls(wishlistItem=wishlist_item, success=d.pop("success", False), unmapped=d)


@dataclass
class WishlistModel:
    wishlistItems: WishlistItemsModel = None
    removeFromWishlist: RemoveFromWishlistModel = None
    addToWishlist: AddToWishlistModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["WishlistModel"], src: Dict[str, Any]) -> "WishlistModel":
        d = src.copy()
        wishlist_items = WishlistItemsModel.from_dict(x) if (x := d.pop("wishlistItems", {})) else None
        remove_from_wishlist = RemoveFromWishlistModel.from_dict(x) if (x := d.pop("removeFromWishlist", {})) else None
        add_to_wishlist = AddToWishlistModel.from_dict(x) if (x := d.pop("addToWishlist", {})) else None
        return cls(
            wishlistItems=wishlist_items,
            removeFromWishlist=remove_from_wishlist,
            addToWishlist=add_to_wishlist,
            unmapped=d,
        )


ProductModel = Dict


@dataclass
class DataModel:
    product: ProductModel = None
    catalog: CatalogModel = None
    wishlist: WishlistModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["DataModel"], src: Dict[str, Any]) -> "DataModel":
        d = src.copy()
        catalog = CatalogModel.from_dict(x) if (x := d.pop("Catalog", {})) else None
        wishlist = WishlistModel.from_dict(x) if (x := d.pop("Wishlist", {})) else None
        return cls(product=d.pop("Product", {}), catalog=catalog, wishlist=wishlist, unmapped=d)


@dataclass
class ErrorModel:
    message: str = None
    correlationId: str = None
    serviceResponse: str = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    def __str__(self):
        return f"{self.correlationId} - {self.message}"

    @classmethod
    def from_dict(cls: Type["ErrorModel"], src: Dict[str, Any]) -> "ErrorModel":
        d = src.copy()
        return cls(
            message=d.pop("message", ""),
            correlationId= d.pop("correlationId", ""),
            serviceResponse=d.pop("serviceResponse", ""),
            unmapped=d
        )


@dataclass
class ExtensionsModel:
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["ExtensionsModel"], src: Dict[str, Any]) -> "ExtensionsModel":
        d = src.copy()
        return cls(unmapped=d)


@dataclass
class ResponseModel:
    data: DataModel = None
    errors: Tuple[ErrorModel, ...] = None
    extensions: ExtensionsModel = None
    unmapped: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["ResponseModel"], src: Dict[str, Any]) -> "ResponseModel":
        d = src.copy()
        data = DataModel.from_dict(x) if (x := d.pop("data", {})) else None
        _errors = d.pop("errors", [])
        errors = tuple(map(ErrorModel.from_dict, _errors))
        extensions = ExtensionsModel.from_dict(x) if (x := d.pop("extensions", {})) else None
        return cls(data=data, errors=errors, extensions=extensions, unmapped=d)



volume_label = 'Rare'
format = 'UDZO'
filesystem = 'HFS+'
size = None
files = ['rare.app']
symlinks = {'Applications': '/Applications'}
icon = 'rare/resources/images/Rare.icns'
background = None
show_status_bar = False
show_tab_view = False
show_pathbar = False
show_sidebar = False
sidebar_width = None
show_icon_preview = True
include_icon_view_settings = 'auto'
include_list_view_settings = 'auto'
arrange_by = None
grid_offset = (0, 0)
grid_spacing = 100
scroll_position = (0, 0)
label_pos = 'bottom'
text_size = 16
icon_size = 128
list_sort_by = None
list_use_relative_dates = None
list_calculate_all_sizes = None
license = {
    'default-language': 'en_US',
    'licenses': {
        'en_US': 'LICENSE'
    },
    'buttons': {
        'en_US': ['English', 'Agree', 'Disagree', 'Print', 'Save', 'License']
    }
}



import os
import platform
from logging import getLogger
from typing import Optional

from PySide6.QtCore import QRunnable, QObject, Signal, QThreadPool, Qt, Slot, QSize
from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import (
    QGroupBox,
    QMessageBox,
    QFrame,
    QHBoxLayout,
    QSizePolicy,
    QLabel,
    QPushButton,
    QFormLayout,
    QComboBox,
)

from rare.lgndr.core import LegendaryCore
from rare.models.game import RareEosOverlay
from rare.shared import RareCore
from rare.ui.components.tabs.games.integrations.eos_widget import Ui_EosWidget
from rare.utils import config_helper as config
from rare.utils.misc import qta_icon
from rare.widgets.elide_label import ElideLabel

logger = getLogger("EpicOverlay")


class CheckForUpdateWorker(QRunnable):
    class CheckForUpdateSignals(QObject):
        update_available = Signal(bool)

    def __init__(self, core: LegendaryCore):
        super(CheckForUpdateWorker, self).__init__()
        self.signals = self.CheckForUpdateSignals()
        self.setAutoDelete(True)
        self.core = core

    def run(self) -> None:
        self.core.check_for_overlay_updates()
        self.signals.update_available.emit(self.core.overlay_update_available)


class EosPrefixWidget(QFrame):
    def __init__(self, overlay: RareEosOverlay, prefix: Optional[str], parent=None):
        super(EosPrefixWidget, self).__init__(parent=parent)
        self.setFrameShape(QFrame.Shape.StyledPanel)
        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)

        self.indicator = QLabel(parent=self)
        self.indicator.setSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Preferred)

        self.prefix_label = ElideLabel(
            prefix.replace(os.path.expanduser("~"), "~") if prefix is not None else overlay.app_title,
            parent=self,
        )
        self.overlay_label = ElideLabel(parent=self)
        self.overlay_label.setDisabled(True)

        self.path_select = QComboBox(self)
        self.path_select.setMaximumWidth(150)
        self.path_select.setMinimumWidth(150)

        self.button = QPushButton(parent=self)
        self.button.setMinimumWidth(150)

        layout = QHBoxLayout(self)
        layout.setContentsMargins(-1, 0, 0, 0)
        layout.addWidget(self.indicator)
        layout.addWidget(self.prefix_label, stretch=2)
        layout.addWidget(self.overlay_label, stretch=3)
        layout.addWidget(self.path_select)
        layout.addWidget(self.button)

        self.overlay = overlay
        self.prefix = prefix

        self.path_select.currentIndexChanged.connect(self.path_changed)
        self.button.clicked.connect(self.action)
        self.overlay.signals.game.installed.connect(self.update_state)
        self.overlay.signals.game.uninstalled.connect(self.update_state)

        self.update_state()

    @Slot(int)
    def path_changed(self, index: int) -> None:
        path = self.path_select.itemData(index, Qt.ItemDataRole.UserRole)
        active_path = os.path.normpath(p) if (p := self.overlay.active_path(self.prefix)) else ""
        if self.overlay.is_enabled(self.prefix) and (path == active_path):
            self.button.setText(self.tr("Disable overlay"))
        else:
            self.button.setText(self.tr("Enable overlay"))

    @Slot()
    def update_state(self) -> None:
        active_path = os.path.normpath(p) if (p := self.overlay.active_path(self.prefix)) else ""

        self.overlay_label.setText(f"<i>{active_path}</i>")
        self.path_select.clear()

        if not self.overlay.is_installed and not self.overlay.available_paths(self.prefix):
            self.setDisabled(True)
            self.indicator.setPixmap(qta_icon("fa.circle-o", color="grey").pixmap(20, 20))
            self.overlay_label.setText(self.overlay.active_path(self.prefix))
            self.button.setText(self.tr("Unavailable"))
            return

        if self.overlay.is_enabled(self.prefix):
            self.indicator.setPixmap(qta_icon("fa.check-circle-o", color="green").pixmap(QSize(20, 20)))
        else:
            self.indicator.setPixmap(qta_icon("fa.times-circle-o", color="red").pixmap(QSize(20, 20)))

        install_path = os.path.normpath(p) if (p := self.overlay.install_path) else ""

        self.path_select.addItem("Auto-detect", "")
        self.path_select.setItemData(0, "Auto-detect", Qt.ItemDataRole.ToolTipRole)
        for path in self.overlay.available_paths(self.prefix):
            path = os.path.normpath(path)
            self.path_select.addItem("Legendary-managed" if path == install_path else "EGL-managed", path)
            self.path_select.setItemData(self.path_select.findData(path), path, Qt.ItemDataRole.ToolTipRole)
        self.path_select.setCurrentIndex(self.path_select.findData(active_path))

        self.setEnabled(self.overlay.state == RareEosOverlay.State.IDLE)

    @Slot()
    def action(self) -> None:
        path = self.path_select.currentData(Qt.ItemDataRole.UserRole)
        active_path = os.path.normpath(p) if (p := self.overlay.active_path(self.prefix)) else ""
        install_path = os.path.normpath(p) if (p := self.overlay.install_path) else ""
        if self.overlay.is_enabled(self.prefix) and (path == active_path):
            if not self.overlay.disable(prefix=self.prefix):
                QMessageBox.warning(
                    self,
                    "Warning",
                    self.tr("Failed to completely disable the active EOS Overlay.{}").format(
                        self.tr(
                            " Since the previous overlay was managed by EGL you can safely ignore this is."
                        )
                        if active_path != install_path
                        else ""
                    ),
                )
        else:
            self.overlay.disable(prefix=self.prefix)
            if not self.overlay.enable(prefix=self.prefix, path=path):
                QMessageBox.warning(
                    self,
                    "Warning",
                    self.tr("Failed to completely enable EOS overlay.{}").format(
                        self.tr(
                            " Since the previous overlay was managed by EGL you can safely ignore this is."
                        )
                        if active_path != install_path
                        else ""
                    ),
                )
        self.update_state()


class EosGroup(QGroupBox):
    def __init__(self, parent=None):
        super(EosGroup, self).__init__(parent=parent)
        self.ui = Ui_EosWidget()
        self.ui.setupUi(self)
        # lk: set object names for CSS properties
        self.ui.install_button.setObjectName("InstallButton")
        self.ui.uninstall_button.setObjectName("UninstallButton")

        self.ui.install_page_layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        self.ui.info_page_layout.setAlignment(Qt.AlignmentFlag.AlignTop)

        self.ui.install_button.setIcon(qta_icon("ri.install-line"))
        self.ui.uninstall_button.setIcon(qta_icon("ri.uninstall-line"))

        self.installed_path_label = ElideLabel(parent=self)
        self.installed_version_label = ElideLabel(parent=self)

        self.ui.info_label_layout.setWidget(0, QFormLayout.ItemRole.FieldRole, self.installed_version_label)
        self.ui.info_label_layout.setWidget(1, QFormLayout.ItemRole.FieldRole, self.installed_path_label)

        self.rcore = RareCore.instance()
        self.core = self.rcore.core()
        self.signals = self.rcore.signals()
        self.overlay = self.rcore.get_overlay()

        self.overlay.signals.widget.update.connect(self.update_state)
        self.overlay.signals.game.installed.connect(self.install_finished)
        self.overlay.signals.game.uninstalled.connect(self.uninstall_finished)

        self.ui.install_button.clicked.connect(self.install_overlay)
        self.ui.update_button.clicked.connect(self.install_overlay)
        self.ui.uninstall_button.clicked.connect(self.uninstall_overlay)

        if self.overlay.is_installed:  # installed
            self.installed_version_label.setText(f"<b>{self.overlay.version}</b>")
            self.installed_path_label.setText(os.path.normpath(self.overlay.install_path))
            self.ui.overlay_stack.setCurrentWidget(self.ui.info_page)
        else:
            self.ui.overlay_stack.setCurrentWidget(self.ui.install_page)
        self.ui.update_button.setEnabled(False)

        self.threadpool = QThreadPool.globalInstance()
        self.worker: Optional[CheckForUpdateWorker] = None

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.check_for_update()
        self.update_prefixes()
        self.update_state()
        super().showEvent(a0)

    @Slot()
    def update_state(self):
        self.ui.install_button.setEnabled(self.overlay.state == RareEosOverlay.State.IDLE)
        self.ui.update_button.setEnabled(self.overlay.state == RareEosOverlay.State.IDLE and self.overlay.has_update)
        self.ui.uninstall_button.setEnabled(self.overlay.state == RareEosOverlay.State.IDLE)

    def update_prefixes(self):
        for widget in self.findChildren(EosPrefixWidget, options=Qt.FindChildOption.FindDirectChildrenOnly):
            widget.deleteLater()

        if platform.system() != "Windows":
            prefixes = config.get_prefixes()
            prefixes = {prefix for prefix, _ in prefixes if config.prefix_exists(prefix)}
            if platform.system() == "Darwin":
                # TODO: add crossover support
                pass
            for prefix in prefixes:
                widget = EosPrefixWidget(self.overlay, prefix)
                self.ui.eos_layout.addWidget(widget)
            logger.debug("Updated prefixes")
        else:
            widget = EosPrefixWidget(self.overlay, None)
            self.ui.eos_layout.addWidget(widget)

    @Slot(bool)
    def check_for_update_finished(self, update_available: bool):
        self.worker = None
        self.ui.update_button.setEnabled(update_available)

    def check_for_update(self):
        self.ui.update_button.setEnabled(False)
        if not self.overlay.is_installed:
            return

        if self.worker is not None:
            return

        self.worker = CheckForUpdateWorker(self.core)
        self.worker.signals.update_available.connect(self.check_for_update_finished)
        QThreadPool.globalInstance().start(self.worker)

    @Slot()
    def install_finished(self):
        if not self.overlay.is_installed:
            logger.error("Something went wrong while installing overlay")
            QMessageBox.warning(self, "Error", self.tr("Something went wrong while installing Overlay"))
            return
        self.ui.overlay_stack.setCurrentWidget(self.ui.info_page)
        self.installed_version_label.setText(f"<b>{self.overlay.version}</b>")
        self.installed_path_label.setText(self.overlay.install_path)
        self.ui.update_button.setEnabled(False)

    @Slot()
    def uninstall_finished(self):
        self.ui.overlay_stack.setCurrentWidget(self.ui.install_page)

    @Slot()
    def install_overlay(self):
        self.overlay.install()

    def uninstall_overlay(self):
        if not self.overlay.is_installed:
            logger.error("No Legendary-managed overlay installation found.")
            self.ui.overlay_stack.setCurrentWidget(self.ui.install_page)
            return
        self.overlay.uninstall()



import platform
import binascii
import shlex
from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import Dict, List, Type, Any


class SteamUser:
    def __init__(self, long_id: str, user: Dict):
        super(SteamUser, self).__init__()
        self._long_id: str = long_id
        self._user = user.copy()

    @property
    def long_id(self) -> int:
        return int(self._long_id)

    @property
    def short_id(self) -> int:
        return self.long_id & 0xFFFFFFFF

    @property
    def account_name(self) -> str:
        return self._user.get("AccountName", "")

    @property
    def persona_name(self) -> str:
        return self._user.get("PersonaName", "")

    @property
    def most_recent(self) -> bool:
        return bool(int(self._user.get("MostRecent", "0")))

    @property
    def last_login(self) -> datetime:
        return datetime.fromtimestamp(float(self._user.get("Timestamp", "0")), timezone.utc)

    @property
    def __dict__(self):
        return dict(
            long_id=self.long_id,
            short_id=self.short_id,
            account_name=self.account_name,
            persona_name=self.persona_name,
            most_recent=self.most_recent,
            last_login=self.last_login,
        )

    def __repr__(self):
        return repr(vars(self))


@dataclass
class SteamShortcut:

    appid: int
    AppName: str
    Exe: str
    StartDir: str
    icon: str
    ShortcutPath: str
    LaunchOptions: str
    IsHidden: bool
    AllowDesktopConfig: bool
    AllowOverlay: bool
    OpenVR: bool
    Devkit: bool
    DevkitGameID: str
    DevkitOverrideAppID: int
    LastPlayTime: int
    FlatpakAppID: str
    tags: Dict = field(default_factory=dict)

    @classmethod
    def from_dict(cls: Type["SteamShortcut"], src: Dict[str, Any]) -> "SteamShortcut":
        d = src.copy()
        tmp = cls(
            appid=d.pop("appid", 0),
            AppName=d.pop("AppName", ""),
            Exe=d.pop("Exe", ""),
            StartDir=d.pop("StartDir", ""),
            icon=d.pop("icon", ""),
            ShortcutPath=d.pop("ShortcutPath", ""),
            LaunchOptions=d.pop("LaunchOptions", ""),
            IsHidden=bool(d.pop("IsHidden", 0)),
            AllowDesktopConfig=bool(d.pop("AllowDesktopConfig", 1)),
            AllowOverlay=bool(d.pop("AllowOverlay", 1)),
            OpenVR=bool(d.pop("OpenVR", 0)),
            Devkit=bool(d.pop("Devkit", 0)),
            DevkitGameID=d.pop("DevkitGameID", ""),
            DevkitOverrideAppID=d.pop("DevkitOverrideAppID", 0),
            LastPlayTime=d.pop("LastPlayTime", 0),
            FlatpakAppID=d.pop("FlatpakAppID", ""),
            tags=d.pop("tags", {}),
        )
        return tmp

    @classmethod
    def create(
        cls: Type["SteamShortcut"],
        app_name: str,
        app_title: str,
        executable: str,
        start_dir: str,
        icon: str,
        launch_options: List[str],
    ) -> "SteamShortcut":
        shortcut = cls.from_dict({})
        shortcut.appid = cls.calculate_appid(app_name)
        shortcut.AppName = app_title
        shortcut.Exe = executable if platform.system() == "Windows" else shlex.quote(executable)
        shortcut.StartDir = start_dir
        shortcut.icon = icon
        shortcut.LaunchOptions = shlex.join(launch_options)
        return shortcut

    @staticmethod
    def calculate_appid(app_name) -> int:
        key = "rare_steam_shortcut_" + app_name
        top = binascii.crc32(str.encode(key, "utf-8")) | 0x80000000
        return (((top << 32) | 0x02000000) >> 32) - 0x100000000

    def shortcut_appid(self) -> int:
        return self.appid

    def coverart_appid(self) -> int:
        return self.shortcut_appid() + 0x100000000

    @property
    def grid_wide(self) -> str:
        return f"{self.coverart_appid()}.png"

    @property
    def grid_tall(self) -> str:
        return f"{self.coverart_appid()}p.png"

    @property
    def game_hero(self) -> str:
        return f"{self.coverart_appid()}_hero.png"

    @property
    def game_logo(self) -> str:
        return f"{self.coverart_appid()}_logo.png"

    @property
    def last_played(self):
        return datetime.fromtimestamp(float(self.LastPlayTime), timezone.utc)

    @property
    def __dict__(self):
        ret = dict(
            shortcut_appid=self.shortcut_appid(), grid_appid=self.coverart_appid(), last_played=self.last_played
        )
        return ret



import json
import os
from logging import getLogger
import shlex
from typing import List, Dict, Iterable
from rare.utils import config_helper as config

from PySide6.QtCore import QSettings

from rare.lgndr.core import LegendaryCore
from rare.models.wrapper import Wrapper, WrapperType
from rare.utils.paths import config_dir

logger = getLogger("Wrappers")


class Wrappers:
    def __init__(self):
        self.__file = os.path.join(config_dir(), "wrappers.json")
        self.__wrappers_dict = {}
        try:
            with open(self.__file, "r", encoding="utf-8") as f:
                self.__wrappers_dict = json.load(f)
        except FileNotFoundError:
            logger.info("%s does not exist", self.__file)
        except json.JSONDecodeError:
            logger.warning("%s is corrupt", self.__file)

        self.__wrappers: Dict[str, Wrapper] = {}
        for wrap_id, wrapper in self.__wrappers_dict.get("wrappers", {}).items():
            self.__wrappers.update({wrap_id: Wrapper.from_dict(wrapper)})

        self.__applists: Dict[str, List[str]] = {}
        for app_name, wrapper_list in self.__wrappers_dict.get("applists", {}).items():
            self.__applists.update({app_name: wrapper_list})

    def import_wrappers(self, core: LegendaryCore, settings: QSettings, app_names: List):
        for app_name in app_names:
            wrappers = self.get_game_wrapper_list(app_name)
            if not wrappers and (commands := settings.value(f"{app_name}/wrapper", [], list)):
                logger.info("Importing wrappers from Rare's config")
                settings.remove(f"{app_name}/wrapper")
                for command in commands:
                    wrapper = Wrapper(command=shlex.split(command))
                    wrappers.append(wrapper)
                    self.set_game_wrapper_list(app_name, wrappers)
                    logger.debug("Imported previous wrappers in %s Rare: %s", app_name, wrapper.name)

            # NOTE: compatibility with Legendary
            if not wrappers and (command := core.lgd.config.get(app_name, "wrapper", fallback="")):
                logger.info("Importing wrappers from legendary's config")
                # no qt wrapper, but legendary wrapper, to have backward compatibility
                # pattern = re.compile(r'''((?:[^ "']|"[^"]*"|'[^']*')+)''')
                # wrappers = pattern.split(command)[1::2]
                wrapper = Wrapper(
                    command=shlex.split(command),
                    name="Imported from Legendary",
                    wtype=WrapperType.LEGENDARY_IMPORT
                )
                wrappers = [wrapper]
                self.set_game_wrapper_list(app_name, wrappers)
                logger.debug("Imported existing wrappers in %s legendary: %s", app_name, wrapper.name)

    @property
    def user_wrappers(self) -> Iterable[Wrapper]:
        return filter(lambda w: w.is_editable, self.__wrappers.values())
        # for wrap in self.__wrappers.values():
        #     if wrap.is_user_defined:
        #         yield wrap

    def get_game_wrapper_string(self, app_name: str) -> str:
        commands = [wrapper.as_str for wrapper in self.get_game_wrapper_list(app_name)]
        return " ".join(commands)

    def get_game_wrapper_list(self, app_name: str) -> List[Wrapper]:
        _wrappers = []
        for wrap_id in self.__applists.get(app_name, []):
            if wrap := self.__wrappers.get(wrap_id, None):
                _wrappers.append(wrap)
        return _wrappers

    def get_game_md5sum_list(self, app_name: str) -> List[str]:
        return self.__applists.get(app_name, [])

    def set_game_wrapper_list(self, app_name: str, wrappers: List[Wrapper]) -> None:
        _wrappers = sorted(wrappers, key=lambda w: w.is_compat_tool)
        for w in _wrappers:
            if (md5sum := w.checksum) in self.__wrappers.keys():
                if w != self.__wrappers[md5sum]:
                    logger.error(
                        "Non-unique md5sum for different wrappers %s, %s",
                        w.name,
                        self.__wrappers[md5sum].name,
                    )
                if w.is_compat_tool:
                    self.__wrappers.update({md5sum: w})
            else:
                self.__wrappers.update({md5sum: w})
        self.__applists[app_name] = [w.checksum for w in _wrappers]
        self.__save_config(app_name)
        self.__save_wrappers()

    def __save_config(self, app_name: str):
        command_string = self.get_game_wrapper_string(app_name)
        config.save_option(app_name, "wrapper", command_string)

    def __save_wrappers(self):
        existing = {wrap_id for wrap_id in self.__wrappers.keys()}
        in_use = {wrap_id for wrappers in self.__applists.values() for wrap_id in wrappers}

        for redudant in existing.difference(in_use):
            del self.__wrappers[redudant]

        self.__wrappers_dict["wrappers"] = self.__wrappers
        self.__wrappers_dict["applists"] = self.__applists

        with open(os.path.join(self.__file), "w+", encoding="utf-8") as f:
            json.dump(self.__wrappers_dict, f, default=lambda o: vars(o), indent=2)


if __name__ == "__main__":
    from pprint import pprint
    from argparse import Namespace

    from rare.utils.compat import steam

    global config_dir
    config_dir = os.getcwd
    global config
    config = Namespace()
    config.set_option = lambda x, y, z: print(x, y, z)
    config.remove_option = lambda x, y: print(x, y)
    config.save_config = lambda: print()

    wr = Wrappers()

    w1 = Wrapper(command=["/usr/bin/w1"], wtype=WrapperType.NONE)
    w2 = Wrapper(command=["/usr/bin/w2"], wtype=WrapperType.COMPAT_TOOL)
    w3 = Wrapper(command=["/usr/bin/w3"], wtype=WrapperType.USER_DEFINED)
    w4 = Wrapper(command=["/usr/bin/w4"], wtype=WrapperType.USER_DEFINED)
    wr.set_game_wrapper_list("testgame", [w1, w2, w3, w4])

    w5 = Wrapper(command=["/usr/bin/w5"], wtype=WrapperType.COMPAT_TOOL)
    wr.set_game_wrapper_list("testgame2", [w2, w1, w5])

    w6 = Wrapper(command=["/usr/bin/w 6", "-w", "-t"], wtype=WrapperType.USER_DEFINED)
    wr.set_game_wrapper_list("testgame", [w1, w2, w3, w6])

    w7 = Wrapper(command=["/usr/bin/w2"], wtype=WrapperType.COMPAT_TOOL)
    wrs = wr.get_game_wrapper_list("testgame")
    wrs.remove(w7)
    wr.set_game_wrapper_list("testgame", wrs)

    game_wrappers = wr.get_game_wrapper_list("testgame")
    pprint(game_wrappers)
    game_wrappers = wr.get_game_wrapper_list("testgame2")
    pprint(game_wrappers)

    for i, tool in enumerate(steam.find_tools()):
        wt = Wrapper(command=tool.command(), name=tool.name, wtype=WrapperType.COMPAT_TOOL)
        wr.set_game_wrapper_list(f"compat_game_{i}", [wt])
        print(wt.as_str)

    for wrp in wr.user_wrappers:
        pprint(wrp)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'landing.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractScrollArea, QApplication, QCheckBox, QFrame,
    QGroupBox, QHBoxLayout, QPushButton, QRadioButton,
    QScrollArea, QSizePolicy, QVBoxLayout, QWidget)

class Ui_LandingWidget(object):
    def setupUi(self, LandingWidget):
        if not LandingWidget.objectName():
            LandingWidget.setObjectName(u"LandingWidget")
        LandingWidget.resize(788, 662)
        LandingWidget.setWindowTitle(u"LandingWidget")
        self.main_layout = QHBoxLayout(LandingWidget)
        self.main_layout.setObjectName(u"main_layout")
        self.left_layout = QVBoxLayout()
        self.left_layout.setObjectName(u"left_layout")

        self.main_layout.addLayout(self.left_layout)

        self.right_layout = QVBoxLayout()
        self.right_layout.setObjectName(u"right_layout")
        self.reset_button = QPushButton(LandingWidget)
        self.reset_button.setObjectName(u"reset_button")

        self.right_layout.addWidget(self.reset_button)

        self.filter_scrollarea = QScrollArea(LandingWidget)
        self.filter_scrollarea.setObjectName(u"filter_scrollarea")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filter_scrollarea.sizePolicy().hasHeightForWidth())
        self.filter_scrollarea.setSizePolicy(sizePolicy)
        self.filter_scrollarea.setFrameShape(QFrame.Shape.NoFrame)
        self.filter_scrollarea.setFrameShadow(QFrame.Shadow.Plain)
        self.filter_scrollarea.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.filter_scrollarea.setSizeAdjustPolicy(QScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.filter_scrollarea.setWidgetResizable(True)
        self.filter_container = QWidget()
        self.filter_container.setObjectName(u"filter_container")
        self.filter_container.setGeometry(QRect(0, 0, 142, 390))
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.filter_container.sizePolicy().hasHeightForWidth())
        self.filter_container.setSizePolicy(sizePolicy1)
        self.filter_container_layout = QVBoxLayout(self.filter_container)
        self.filter_container_layout.setObjectName(u"filter_container_layout")
        self.filter_container_layout.setContentsMargins(0, 0, 3, 0)
        self.price_group = QGroupBox(self.filter_container)
        self.price_group.setObjectName(u"price_group")
        self.price_layout = QVBoxLayout(self.price_group)
        self.price_layout.setObjectName(u"price_layout")
        self.none_price = QRadioButton(self.price_group)
        self.none_price.setObjectName(u"none_price")
        self.none_price.setChecked(True)

        self.price_layout.addWidget(self.none_price)

        self.free_button = QRadioButton(self.price_group)
        self.free_button.setObjectName(u"free_button")

        self.price_layout.addWidget(self.free_button)

        self.under10 = QRadioButton(self.price_group)
        self.under10.setObjectName(u"under10")

        self.price_layout.addWidget(self.under10)

        self.under20 = QRadioButton(self.price_group)
        self.under20.setObjectName(u"under20")

        self.price_layout.addWidget(self.under20)

        self.under30 = QRadioButton(self.price_group)
        self.under30.setObjectName(u"under30")

        self.price_layout.addWidget(self.under30)

        self.above = QRadioButton(self.price_group)
        self.above.setObjectName(u"above")

        self.price_layout.addWidget(self.above)

        self.on_discount = QCheckBox(self.price_group)
        self.on_discount.setObjectName(u"on_discount")

        self.price_layout.addWidget(self.on_discount)


        self.filter_container_layout.addWidget(self.price_group)

        self.platform_group = QGroupBox(self.filter_container)
        self.platform_group.setObjectName(u"platform_group")
        self.platfrom_layout = QVBoxLayout(self.platform_group)
        self.platfrom_layout.setObjectName(u"platfrom_layout")

        self.filter_container_layout.addWidget(self.platform_group)

        self.genre_group = QGroupBox(self.filter_container)
        self.genre_group.setObjectName(u"genre_group")
        self.genre_layout = QVBoxLayout(self.genre_group)
        self.genre_layout.setObjectName(u"genre_layout")

        self.filter_container_layout.addWidget(self.genre_group)

        self.type_group = QGroupBox(self.filter_container)
        self.type_group.setObjectName(u"type_group")
        self.type_layout = QVBoxLayout(self.type_group)
        self.type_layout.setObjectName(u"type_layout")

        self.filter_container_layout.addWidget(self.type_group)

        self.others_group = QGroupBox(self.filter_container)
        self.others_group.setObjectName(u"others_group")
        self.others_layout = QVBoxLayout(self.others_group)
        self.others_layout.setObjectName(u"others_layout")

        self.filter_container_layout.addWidget(self.others_group)

        self.filter_scrollarea.setWidget(self.filter_container)

        self.right_layout.addWidget(self.filter_scrollarea)


        self.main_layout.addLayout(self.right_layout)

        self.main_layout.setStretch(0, 1)

        self.retranslateUi(LandingWidget)
    # setupUi

    def retranslateUi(self, LandingWidget):
        self.reset_button.setText(QCoreApplication.translate("LandingWidget", u"Reset filters", None))
        self.price_group.setTitle(QCoreApplication.translate("LandingWidget", u"Price", None))
        self.none_price.setText(QCoreApplication.translate("LandingWidget", u"None", None))
        self.free_button.setText(QCoreApplication.translate("LandingWidget", u"Free", None))
        self.under10.setText(QCoreApplication.translate("LandingWidget", u"Under 10", None))
        self.under20.setText(QCoreApplication.translate("LandingWidget", u"Under 20", None))
        self.under30.setText(QCoreApplication.translate("LandingWidget", u"Under 30", None))
        self.above.setText(QCoreApplication.translate("LandingWidget", u"14.99 and above", None))
        self.on_discount.setText(QCoreApplication.translate("LandingWidget", u"Discount", None))
        self.platform_group.setTitle(QCoreApplication.translate("LandingWidget", u"Platform", None))
        self.genre_group.setTitle(QCoreApplication.translate("LandingWidget", u"Genre", None))
        self.type_group.setTitle(QCoreApplication.translate("LandingWidget", u"Type", None))
        self.others_group.setTitle(QCoreApplication.translate("LandingWidget", u"Other tags", None))
        pass
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'browser_login.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFormLayout, QHBoxLayout, QLabel,
    QLayout, QLineEdit, QPushButton, QSizePolicy,
    QSpacerItem, QVBoxLayout, QWidget)

class Ui_BrowserLogin(object):
    def setupUi(self, BrowserLogin):
        if not BrowserLogin.objectName():
            BrowserLogin.setObjectName(u"BrowserLogin")
        BrowserLogin.resize(320, 243)
        BrowserLogin.setWindowTitle(u"BrowserLogin")
        self.main_layout = QVBoxLayout(BrowserLogin)
        self.main_layout.setObjectName(u"main_layout")
        self.title_label = QLabel(BrowserLogin)
        self.title_label.setObjectName(u"title_label")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.title_label.sizePolicy().hasHeightForWidth())
        self.title_label.setSizePolicy(sizePolicy)
        font = QFont()
        font.setBold(True)
        self.title_label.setFont(font)

        self.main_layout.addWidget(self.title_label)

        self.form_layout = QFormLayout()
        self.form_layout.setObjectName(u"form_layout")
        self.form_layout.setSizeConstraint(QLayout.SetFixedSize)
        self.form_layout.setFieldGrowthPolicy(QFormLayout.AllNonFixedFieldsGrow)
        self.form_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.form_layout.setFormAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignVCenter)
        self.open_button = QPushButton(BrowserLogin)
        self.open_button.setObjectName(u"open_button")

        self.form_layout.setWidget(0, QFormLayout.LabelRole, self.open_button)

        self.link_layout = QHBoxLayout()
        self.link_layout.setObjectName(u"link_layout")
        self.link_text = QLineEdit(BrowserLogin)
        self.link_text.setObjectName(u"link_text")
        self.link_text.setText(u"")
        self.link_text.setReadOnly(True)
        self.link_text.setPlaceholderText(u"")

        self.link_layout.addWidget(self.link_text)

        self.copy_button = QPushButton(BrowserLogin)
        self.copy_button.setObjectName(u"copy_button")
        self.copy_button.setText(u"")

        self.link_layout.addWidget(self.copy_button)

        self.link_layout.setStretch(0, 1)

        self.form_layout.setLayout(0, QFormLayout.ItemRole.FieldRole, self.link_layout)

        self.sid_label = QLabel(BrowserLogin)
        self.sid_label.setObjectName(u"sid_label")
        self.sid_label.setText(u"authorizationCode")
        self.sid_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.form_layout.setWidget(1, QFormLayout.LabelRole, self.sid_label)

        self.status_label = QLabel(BrowserLogin)
        self.status_label.setObjectName(u"status_label")
        font1 = QFont()
        font1.setItalic(True)
        self.status_label.setFont(font1)
        self.status_label.setText(u"")

        self.form_layout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.status_label)


        self.main_layout.addLayout(self.form_layout)

        self.vscaper = QSpacerItem(20, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.main_layout.addItem(self.vscaper)

        self.info_label = QLabel(BrowserLogin)
        self.info_label.setObjectName(u"info_label")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.info_label.sizePolicy().hasHeightForWidth())
        self.info_label.setSizePolicy(sizePolicy1)
        self.info_label.setWordWrap(True)

        self.main_layout.addWidget(self.info_label)


        self.retranslateUi(BrowserLogin)
    # setupUi

    def retranslateUi(self, BrowserLogin):
        self.title_label.setText(QCoreApplication.translate("BrowserLogin", u"Login through browser", None))
        self.open_button.setText(QCoreApplication.translate("BrowserLogin", u"Open Browser", None))
        self.info_label.setText(QCoreApplication.translate("BrowserLogin", u"<i>Click the <strong>Open Browser</strong> button to open the login page in your web browser or copy the link and paste it in any web browser. After logging in using the browser, copy the text in the quotes after </i><code><b>authorizationCode</b></code><i> in the same line into the empty input above.<br><br><strong>DO NOT SHARE THE INFORMATION IN THE BROWSER PAGE WITH ANYONE IN ANY FORM (TEXT OR SCREENSHOT)!</strong></i>", None))
        pass
    # retranslateUi

